/****************************************************************************
  Copyright (c) 2006, Dell Inc
  All rights reserved.
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:
  - Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer.
    
  - Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution. 
  - Neither the name of Dell Inc nor the names of its contributors
    may be used to endorse or promote products derived from this software 
    without specific prior written permission. 
  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE. 
 
 
*****************************************************************************/

#ifndef __COMMONSSIPMI_H_INCLUDED__
#define __COMMONSSIPMI_H_INCLUDED__

/* Defines */

#define _throw      goto

#define _IN
#define _OUT
#define _INOUT


#define TO_COOKED                           (1)
#define TO_RAW                              (2)

#define IPMI_INTEL_OEM_IMPLEMENTATION       (1)


typedef unsigned short booln;


/* status defines */
#define COMMONSS_STATUS_SUCCESS             (0)

/* Severity defines */
#define CSS_SEVERITY_UNKNOWN                 (0)
#define CSS_SEVERITY_INFORMATIONAL           (1)
#define CSS_SEVERITY_NORMAL                  (2)
#define CSS_SEVERITY_WARNING                 (3)
#define CSS_SEVERITY_CRITICAL                (4)
#define CSS_SEVERITY_NONRECOVERABLE          (5)

/* IPMI Defined Sensor Types */
#define IPMI_SENSOR_TEMP               (0x01)
#define IPMI_SENSOR_VOLT               (0x02)
#define IPMI_SENSOR_CURRENT            (0x03)
#define IPMI_SENSOR_FAN                (0x04)
#define IPMI_SENSOR_INTRUSION          (0x05)
#define IPMI_SENSOR_SECURE_MODE        (0x06)
#define IPMI_SENSOR_PROCESSOR          (0x07)
#define IPMI_SENSOR_PS                 (0x08)
#define IPMI_SENSOR_POWER_UNIT         (0x09)  
#define IPMI_SENSOR_COOLING_UNIT       (0x0a)
#define IPMI_SENSOR_MEMORY             (0x0c)
#define IPMI_SENSOR_DRIVE_SLOT         (0x0d)
#define IPMI_SENSOR_MEM_RESIZE         (0x0e)
#define IPMI_SENSOR_POST_ERROR         (0x0f)
#define IPMI_SENSOR_EVENT_LOGGING      (0x10)
#define IPMI_SENSOR_WDOG               (0x11)
#define IPMI_SENSOR_SYSTEM_EVENT       (0x12)
#define IPMI_SENSOR_CRIT_EVENT         (0x13)
#define IPMI_SENSOR_POWER_BUTTON       (0x14)
#define IPMI_SENSOR_VRM                (0x15)
#define IPMI_SENSOR_MC                 (0x16)
#define IPMI_SENSOR_CHIP_SET           (0x19)
#define IPMI_SENSOR_CI                 (0x1b)  
#define IPMI_SENSOR_SYSTEM_BOOT        (0x1d)
#define IPMI_SENSOR_BOOT_ERROR         (0x1e)
#define IPMI_SENSOR_OS_BOOT            (0x1f)
#define IPMI_SENSOR_OS_CRITICAL_STOP   (0x20)
#define IPMI_SENSOR_SLOT               (0x21)
#define IPMI_SENSOR_ACPI               (0x22)
#define IPMI_SENSOR_WDOG2              (0x23)
#define IPMI_SENSOR_PLATFORM_ALERT     (0x24)
#define IPMI_SENSOR_ENTITY_PRESENCE    (0x25)
#define IPMI_SENSOR_ASIC_MONITOR       (0x26)
#define IPMI_SENSOR_LAN                (0x27)
#define IPMI_SENSOR_MGMT_SYS_HEALTH    (0x28)
#define IPMI_SENSOR_BATTERY            (0x29)
#define IPMI_SENSOR_SESSION_AUDIT      (0x2A)
#define IPMI_SENSOR_VERSION_CHANGE     (0x2B)
#define IPMI_SENSOR_FRU_STATE          (0x2C)


#define IPMI_SENSOR_SYS_DEGRADE_STATUS (0xC0)

/* IPMI Entity ID definitions */
#define IPMI_ENTITY_ID_OTHER                (0x01)
#define IPMI_ENTITY_ID_UNKNOWN              (0x02)
#define IPMI_ENTITY_ID_PROCESSOR            (0x03)
#define IPMI_ENTITY_ID_DISK                 (0x04)
#define IPMI_ENTITY_ID_PBAY                 (0x05)
#define IPMI_ENTITY_ID_SYS_MGMT             (0x06)
#define IPMI_ENTITY_ID_SYS_BOARD            (0x07)
#define IPMI_ENTITY_ID_MEM_MODULE           (0x08)
#define IPMI_ENTITY_ID_PROC_MODULE          (0x09)
#define IPMI_ENTITY_ID_POWER_SUPPLY         (0x0a)
#define IPMI_ENTITY_ID_ADD_IN_CARD          (0x0b)
#define IPMI_ENTITY_ID_FRONT_PANEL          (0x0c)
#define IPMI_ENTITY_ID_BACK_PANEL           (0x0d)
#define IPMI_ENTITY_ID_PS_BOARD             (0x0e)
#define IPMI_ENTITY_ID_DRIVE_BP             (0x0f)
#define IPMI_ENTITY_ID_SIE_BOARD            (0x10)
#define IPMI_ENTITY_ID_OTHER_BOARD          (0x11)
#define IPMI_ENTITY_ID_PROC_BOARD           (0x12)
#define IPMI_ENTITY_ID_POWER_UNIT           (0x13)
#define IPMI_ENTITY_ID_VRM                  (0x14)
#define IPMI_ENTITY_ID_COOLING_DEVICE       (0x1D)
#define IPMI_ENTITY_ID_COOLING_UNIT         (0x1E)
#define IPMI_ENTITY_ID_CABLE_IC             (0x1F)
#define IPMI_ENTITY_ID_MEMORY_DEVICE        (0x20)
#define IPMI_ENTITY_ID_BATTERY_DEVICE       (0x28)

/* IPMI SDR Type */
#define SDR_FULL_SENSOR                     (0x01) /* 0.9+ (internal sensor with thresholds for 0.9)*/
#define SDR_COMPACT_SENSOR                  (0x02) /* 0.9+ (digital/discrete no thresholds for 0.9)*/
#define SDR_ENTITY_ASSOC                    (0x08) /* 1.0+*/
#define SDR_DEVICE_ENTITY_ASSOC             (0x09) /* 1.5+*/
#define SDR_GENERIC_DEV_LOCATOR             (0x10) /* 0.9+ (IPMB device locator for 0.9)*/
#define SDR_FRU_DEV_LOCATOR                 (0x11) /* 1.0+*/
#define SDR_MGMT_CTRL_DEV_LOCATOR           (0x12) /* 1.0+*/
#define SDR_MGMT_CTRL_CONFIRM               (0x13) /* 1.0+*/
#define SDR_BMC_MSSG_CHANNEL_INFO           (0x14) /* 1.0+*/
#define SDR_OEM_RECORD                      (0xC0) /* 0.9+*/

/* reading type defines */
#define IPMI_READING_TYPE_THRESHOLD             (0x01)
#define IPMI_READING_TYPE_DISCRETE              (0x02)
#define IPMI_READING_TYPE_DIG_DISCRETE3         (0x03)
#define IPMI_READING_TYPE_DIG_DISCRETE4         (0x04)
#define IPMI_READING_TYPE_DIG_DISCRETE5         (0x05)
#define IPMI_READING_TYPE_DIG_DISCRETE6         (0x06)
#define IPMI_READING_TYPE_DIG_DISCRETE7         (0x07)
#define IPMI_READING_TYPE_DIG_DISCRETE8         (0x08)
#define IPMI_READING_TYPE_DIG_DISCRETE9         (0x09)
#define IPMI_READING_TYPE_DISCRETE0A            (0x0A)
#define IPMI_READING_TYPE_REDUNDANCY            (0x0B)
#define IPMI_READING_TYPE_DISCRETE0C            (0x0C)
#define IPMI_READING_TYPE_SPEC_OFFSET           (0x6f)


/* event trigger types */
#define IPMI_LNC_LOW                                    (0x00)
#define IPMI_LNC_HIGH                                   (0x01)
#define IPMI_LC_LOW                                     (0x02)
#define IPMI_LC_HIGH                                    (0x03)
#define IPMI_LNR_LOW                                    (0x04)
#define IPMI_LNR_HIGH                                   (0x05)
#define IPMI_UNC_LOW                                    (0x06)
#define IPMI_UNC_HIGH                                   (0x07)
#define IPMI_UC_LOW                                     (0x08)
#define IPMI_UC_HIGH                                    (0x09)
#define IPMI_UNR_LOW                                    (0x0a)
#define IPMI_UNR_HIGH                                   (0x0b)

#define IPMI_UNR_THRESHOLD_MASK                         BIT(5)
#define IPMI_UC_THRESHOLD_MASK                          BIT(4)
#define IPMI_UNC_THRESHOLD_MASK                         BIT(3)
#define IPMI_LNR_THRESHOLD_MASK                         BIT(2)
#define IPMI_LC_THRESHOLD_MASK                          BIT(1)
#define IPMI_LNC_THRESHOLD_MASK                         BIT(0)



/* IPMI Strcutures */
#pragma pack(1)

typedef struct _IPMISDRHeader 
{
    /* record ID of the SDR */
    unsigned short recordID;
    /* SDR version info */
    unsigned char sdrVer;
    /* type of SDR record  */
    unsigned char recordType;
    /* SDR record lenght */
    unsigned char recordLength;
} IPMISDRHeader;


typedef struct _IPMISDRType1 
{
    unsigned char ownerID;
    unsigned char ownerLUN;
    unsigned char sensorNum;
    unsigned char entityID;
    unsigned char entityInstance;
    unsigned char sensorInit;
    unsigned char sensorCaps;
    unsigned char sensorType;
    unsigned char readingType;
    unsigned short triggerLTRMask; 
    unsigned short triggerUTRMask;
    unsigned short readingMask;     
    unsigned char units1;
    unsigned char units2;
    unsigned char units3;
    unsigned char linearization;
    unsigned char m;
    unsigned char tolerance;
    unsigned char b;
    unsigned char accuracy;
    unsigned char accuracyExp;
    unsigned char rbExp;
    unsigned char analogChars;
    unsigned char nominalReading;
    unsigned char normalMax;
    unsigned char normalMin;
    unsigned char sensorMax;
    unsigned char sensorMin;
    unsigned char upperNR;
    unsigned char upperC;
    unsigned char upperNC;
    unsigned char lowerNR;
    unsigned char lowerC;
    unsigned char lowerNC;
    unsigned char positiveHystersis;
    unsigned char negativeHysterisis;
    unsigned char reserved1;
    unsigned char reserved2;
    unsigned char OEM;
    unsigned char typeLengthCode;
    unsigned char sensorName[16]; 
} IPMISDRType1; 


typedef struct _IPMISDRType2 
{
    unsigned char ownerID;
    unsigned char ownerLUN;
    unsigned char sensorNum;
    unsigned char entityID;
    unsigned char entityInstance;
    unsigned char sensorInit;
    unsigned char sensorCaps;
    unsigned char sensorType;
    unsigned char readingType;
    unsigned short triggerLTRMask;  
    unsigned short triggerUTRMask;
    unsigned short readingMask;     
    unsigned char units1;
    unsigned char units2;
    unsigned char units3;
    unsigned char recordSharing1;
    unsigned char recordSharing2;
    unsigned char positiveHystersis;
    unsigned char negativeHysterisis;
    unsigned char reserved1;
    unsigned char reserved2;
    unsigned char reserved3;
    unsigned char OEM;
    unsigned char typeLengthCode;
    unsigned char sensorName[16];   
} IPMISDRType2; 


typedef struct _IPMISDRType8 
{
    unsigned char containerEntityID;
    unsigned char containerEntityInstance;
    unsigned char flags;
    unsigned char containedEntityID1R1;     
    unsigned char containedEntityInstance1R1; 

    unsigned char containedEntityID2R1;     
    unsigned char containedEntityInstance2R1; 
    unsigned char containedEntityID3R2;     
    unsigned char containedEntityInstance3R2; 
    unsigned char containedEntityID4R2;     
    unsigned char containedEntityInstance4R2; 
} IPMISDRType8; 


typedef struct _IPMIDellOem 
{
    unsigned char ownerID;
    unsigned char sensorNum;
    unsigned char oemString[1];
} IPMIDellOem;


typedef struct _IPMISDRTypeC0
{
    unsigned char vendorID[3];
    union {
        unsigned char oem[1];   
        IPMIDellOem dellOem;
    } OemType;
} IPMISDRTypeC0; 


/* FRU device locator record */
typedef struct _IPMISdrType11 {
    unsigned char devAccessAddr;
    unsigned char fruSlaveAddr;
    unsigned char privateBusId;
    unsigned char reserved[2];
    unsigned char devType;
    unsigned char devTypeModifier;
    unsigned char fruEntityId;
    unsigned char fruEntityInst;
    unsigned char oemByte;
    unsigned char devIdStrTypeLen;
    unsigned char devString[16];
} IPMISdrType11;


/* MGMT Controller device locator record */
typedef struct _IPMISdrType12 {
    unsigned char devAccessAddr;
    unsigned char channelNumber;
    unsigned char powerState;
    unsigned char devCaps;
    unsigned char reserved[3];
    unsigned char fruEntityID;
    unsigned char fruEntityInst;
    unsigned char oemByte;
    unsigned char devIdStrTypeLen;
    unsigned char devString[16];
} IPMISdrType12;

typedef union _IPMISDRType 
{
    IPMISDRType1 type1;
    IPMISDRType2 type2;
    IPMISDRType8 type8;
    IPMISdrType11 type11;
    IPMISdrType12 type12;
    IPMISDRTypeC0 typeC0;
} IPMISDRType;

/* IPMISDR structure represents all the types of SDR's
 using unions for each SDR record type */
typedef struct _IPMISDR
{
    IPMISDRHeader header;
    IPMISDRType type;
} IPMISDR;


typedef struct _IPMISELEntry 
{
    unsigned short recordID;
    unsigned char recordType;
    unsigned char timeStamp[4];
    unsigned char generatorID1;
    unsigned char generatorID2;
    unsigned char evmRev;
    unsigned char sensorType;
    unsigned char sensorNum;
    unsigned char eventDirType;
    unsigned char eventData1;
    unsigned char eventData2;
    unsigned char eventData3;
} IPMISELEntry;


#pragma pack()


#endif /* __COMMONSSIPMI_H_INCLUDED__ */


