/*
	Copyright (c) 2007 Dell Inc
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <errno.h>
#include <unistd.h>
#include <signal.h>
#include <ctype.h>
#include <limits.h>						// _MIN and _MAX
#include <ipmitool/ipmi.h>
#include <ipmitool/ipmi_intf.h>
#include <ipmitool/helper.h>
#include <ipmitool/log.h>
#include <ipmitool/ipmi_sel.h>
//#include <ipmitool/ipmi_sdr.h>
//#include <ipmitool/ipmi_strings.h>
//#include <ipmitool/ipmi_channel.h>
#include <ipmitool/ipmi_delloem.h>
//#include <ipmitool/ipmi_raw.h>
#include <ipmitool/ipmi_fru.h>
#include <ipmitool/ipmi_sdr.h>
#include <ipmitool/ipmi_mc.h>
#include <ipmitool/ipmi_sensor.h>
#include <ipmitool/ipmi_sel.h>
//------------ipmi headers------------------
#include <ipmitool/cssstat.h>
#include <ipmitool/cssipmi.h>
#include <ipmitool/cssapi.h>
#include <ipmitool/csssupt.h>
#include <ipmitool/cssdprod.h>
#include <ipmitool/cmnslf.h>
//--------------time header-----------------
#include <time.h>


extern int read_fru_area(struct ipmi_intf * intf, struct fru_info *fru, 
		uint8_t id,uint32_t offset, uint32_t length, uint8_t *frubuf);
extern char *get_fru_area_str(uint8_t * data, uint32_t * offset);

static void
ipmi_sysinfo_print_board(struct ipmi_intf * intf, struct fru_info * fru,
		     uint8_t id, uint32_t offset, uint8_t ipmi_version);
static int ipmi_sysinfo_get_tag(struct ipmi_intf * intf, char* tagstring, uint8_t tag_type, uint8_t length);
extern int  ipmi_sel_get(struct ipmi_intf * intf, uint16_t id);
static int
ipmi_sysinfo_fru(struct ipmi_intf * intf, uint8_t id, uint8_t ipmi_version);
static char*  ipmi_get_bib_info(uint8_t * fru_data, uint8_t bib_type);
uint16_t ipmi_sel_get_record(struct ipmi_intf * intf, uint16_t curr_id, uint8_t* sel_entry);
int
ipmi_sdr_reserve(struct ipmi_intf *intf, uint16_t * reserve_id);
int ipmi_sdr_get(struct ipmi_intf *intf, uint16_t reserveId, uint8_t* datapt, uint16_t recordId);
SDRType * ipmi_sdr_get_first(void* x);
SDRType * ipmi_sdr_get_next(SDRType *w, void *x);
int ipmi_sel_sysinfo(struct ipmi_intf *intf, DECODING_METHOD *method);
int ipmi_sel_print(struct ipmi_intf * intf, uint16_t count, int printOption);
static int
ipmi_sel_clear(struct ipmi_intf * intf);
static uint16_t
ipmi_sel_reserve(struct ipmi_intf * intf);
static int ipmi_sensor_print(SDRType* pSdr, int* pSensorIndex, struct ipmi_intf * intf);
int
ipmi_sensor_get_sensor_reading(unsigned char sensorOwner, unsigned char sensorNumber,
							   IPMISensorReadingType* sensorReadingData,
							   void* x);
int
ipmi_sensor_get_sensor_thresholds(unsigned char sensorOwner, unsigned char sensorNumber,
								  IPMISensorThresholdType* pThrData,
								  void* x);
SDRType* ipmi_sdr_get_test(struct ipmi_intf *intf, uint16_t reserveId, uint8_t* datapt, uint16_t recordId);
SDRType* ipmi_sdr_get_test2(struct ipmi_intf *intf, uint16_t reserveId, uint8_t* datapt, uint16_t recordId);
SDRType* ipmi_sdr_get_test3(struct ipmi_intf *intf, uint16_t reserveId, uint8_t* datapt, uint16_t recordId);
static int ipmi_powermgmt(struct ipmi_intf* intf);
static void
ipmi_time_to_str(time_t rawTime, char* strTime);
static int
ipmi_sel_get_info(struct ipmi_intf * intf);
static char *
ipmi_sel_timestamp(uint32_t stamp);
static int make_int(const char *str, int *value);
static int
ipmi_sel_get_time(struct ipmi_intf * intf);
static int
ipmi_sel_set_time(struct ipmi_intf * intf, const char * time_string);
static
struct ipmi_rs *
ipmi_sensor_set_sensor_thresholds(struct ipmi_intf *intf,
				  uint8_t sensor,
				  uint8_t threshold, uint8_t setting);
static int
__ipmi_sensor_set_threshold(struct ipmi_intf *intf,
			    uint8_t num, uint8_t mask, uint8_t setting);
static int ipmi_build_sdr_first(struct ipmi_intf *intf, uint8_t *data);
int ipmi_build_sdr_next(struct ipmi_intf *intf, uint8_t *sdr_in, uint8_t *data);
static int ipmi_sdr_build_table(struct ipmi_intf *intf);
SDRType* sdr_get_first();
SDRType* sdr_get_next(SDRType* pPreviousSDR);


uint8_t  data[64];

static const struct valstr threshold_vals[] = {
	{UPPER_NON_RECOV_SPECIFIED, "Upper Non-Recoverable"},
	{UPPER_CRIT_SPECIFIED, "Upper Critical"},
	{UPPER_NON_CRIT_SPECIFIED, "Upper Non-Critical"},
	{LOWER_NON_RECOV_SPECIFIED, "Lower Non-Recoverable"},
	{LOWER_CRIT_SPECIFIED, "Lower Critical"},
	{LOWER_NON_CRIT_SPECIFIED, "Lower Non-Critical"},
	{0x00, NULL},
};

typedef struct _SDRList
{
	uint8_t data[MAX_SDR_SIZE];
	uint8_t bIsLast;
} SDRList;


SDRList SDR[MAX_SDR_ENTRIES];
uint16_t currentSDR;
uint16_t totalSDRCount;


// Dell OEM LCD command
int
ipmi_lcd_get_configure_command (struct ipmi_intf * intf, uint8_t *command)
{
	struct ipmi_rs * rsp = NULL;
	struct ipmi_rq req = {0};
	uint8_t data[4];

	req.msg.netfn = IPMI_NETFN_APP;
	req.msg.cmd = IPMI_GET_SYS_INFO;
	req.msg.data_len = 4;
	req.msg.data = data;
	data[0] = 0;
	data[1] = IPMI_DELL_LCD_CONFIG_SELECTOR;
	data[2] = 0;
	data[3] = 0;

	rsp = intf->sendrecv(intf, &req);
	if (rsp == NULL) {
		lprintf(LOG_ERR, " Error getting LCD configuration");
		return -1;
	} else if (rsp->ccode > 0) {
		lprintf(LOG_ERR, " Error getting LCD configuration: %s",
			val2str(rsp->ccode, completion_code_vals));
		return -1;
	}

	// rsp->data[0] is the rev
	*command = rsp->data[1];

	return 0;
}



int
ipmi_lcd_set_configure_command (struct ipmi_intf * intf, int command)
{
#define LSCC_DATA_LEN 2

	struct ipmi_rs * rsp = NULL;
	struct ipmi_rq req = {0};
	uint8_t data[2];

	req.msg.netfn = IPMI_NETFN_APP;
	req.msg.cmd = IPMI_SET_SYS_INFO;
	req.msg.data_len = 2;
	req.msg.data = data;
	data[0] = IPMI_DELL_LCD_CONFIG_SELECTOR;
	data[1] = command;			// command - custom, default, none

	rsp = intf->sendrecv(intf, &req);
	if (rsp == NULL) {
		lprintf(LOG_ERR, " Error setting LCD configuration");
		return -1;
	} else if (rsp->ccode > 0) {
		lprintf(LOG_ERR, " Error setting LCD configuration: %s",
			val2str(rsp->ccode, completion_code_vals));

		return -1;
	}

	return 0;
}

static
ipmi_lcd_get_platform_model_name (struct ipmi_intf * intf, char* lcdstring, uint8_t max_length, uint8_t field_type)
{
	struct ipmi_rs * rsp = NULL;
	struct ipmi_rq req = {0};
	uint8_t data[4];
	IPMI_DELL_LCD_STRING * lcdstringblock;
	int lcdstring_len = 0;
	int bytes_copied = 0;

//	printf("hex value of 'test' = %x\n", "test");
//	char lcdstring[IPMI_DELL_LCD_STRING_LENGTH_MAX+1] = {0};
	int ii;

	for (ii = 0; ii < 4; ii++)
	{
		int bytes_to_copy;

		req.msg.netfn = IPMI_NETFN_APP;
		req.msg.cmd = IPMI_GET_SYS_INFO;
		req.msg.data_len = 4;
		req.msg.data = data;
		data[0] = 0;				// get parameter
		data[1] = field_type;
//		data[1] = 3;/*to get os name*/
		data[2] = ii;
		data[3] = 0;

//		printf("Entering print function\n");
//		printf("ii = %d\n", ii);

		rsp = intf->sendrecv(intf, &req);
		if (rsp == NULL) {
			lprintf(LOG_ERR, " Error getting platform model name");
//			return NULL;
		} else if (rsp->ccode > 0) {
			lprintf(LOG_ERR, " Error getting platform model name: %s",
				val2str(rsp->ccode, completion_code_vals));
//			printf("sniff ccode = %x\n", rsp->ccode);
//			return NULL;
		}

		lcdstringblock = (IPMI_DELL_LCD_STRING *) (void *) rsp->data;
//		printf("Encoding = %X\n",lcdstringblock->lcd_string.selector_0_string.encoding);
		
		// first block is different - 14 bytes
		if (0 == ii) {
//			printf("Entered first 'if'\n");
			lcdstring_len = lcdstringblock->lcd_string.selector_0_string.length;
//			printf("lcdstring_len = %d\nmaxlen = %d\n", lcdstring_len,max_length);

			if (lcdstring_len < 1 || lcdstring_len > max_length)
				break;

			bytes_to_copy = MIN(lcdstring_len, IPMI_DELL_LCD_STRING1_SIZE);
			memcpy (lcdstring, lcdstringblock->lcd_string.selector_0_string.data, bytes_to_copy);
//			printf("Test%d = %s\n", ii, lcdstring);
		} else {
			int string_offset;

			bytes_to_copy = MIN(lcdstring_len - bytes_copied, IPMI_DELL_LCD_STRINGN_SIZE);
			if (bytes_to_copy < 1)
				break;
			string_offset = IPMI_DELL_LCD_STRING1_SIZE + IPMI_DELL_LCD_STRINGN_SIZE * (ii-1);
			memcpy (lcdstring+string_offset, lcdstringblock->lcd_string.selector_n_data, bytes_to_copy);
//			printf("Test%d = %s\n", ii, lcdstring);
		}
		

		bytes_copied += bytes_to_copy;
		if (bytes_copied >= lcdstring_len)
			break;
	}
//	printf("Last test string = %s\n", lcdstring);

//	return lcdstring;
}

static int
ipmi_lcd_get_single_line_text (struct ipmi_intf * intf, char* lcdstring, uint8_t max_length)
{
	struct ipmi_rs * rsp = NULL;
	struct ipmi_rq req = {0};
	uint8_t data[4];
	IPMI_DELL_LCD_STRING * lcdstringblock;
	int lcdstring_len = 0;
	int bytes_copied = 0;
//	char lcdstring[IPMI_DELL_LCD_STRING_LENGTH_MAX+1] = {0};
	int ii;

	for (ii = 0; ii < 4; ii++) {
		int bytes_to_copy;

		req.msg.netfn = IPMI_NETFN_APP;
		req.msg.cmd = IPMI_GET_SYS_INFO;
		req.msg.data_len = 4;
		req.msg.data = data;
		data[0] = 0;				// get parameter
		data[1] = IPMI_DELL_LCD_STRING_SELECTOR;
		data[2] = ii;				// block selector
		data[3] = 00;				// set selector (n/a)

		rsp = intf->sendrecv(intf, &req);
		if (rsp == NULL) {
			lprintf(LOG_ERR, " Error getting text data");
			return -1;
		} else if (rsp->ccode > 0) {
			lprintf(LOG_ERR, " Error getting text data: %s",
				val2str(rsp->ccode, completion_code_vals));
			return -1;
		}

		lcdstringblock = (IPMI_DELL_LCD_STRING *) (void *) rsp->data;
		
		// first block is different - 14 bytes
		if (0 == ii) {
			lcdstring_len = lcdstringblock->lcd_string.selector_0_string.length;

			if (lcdstring_len < 1 || lcdstring_len > max_length)
				break;

			bytes_to_copy = MIN(lcdstring_len, IPMI_DELL_LCD_STRING1_SIZE);
			memcpy (lcdstring, lcdstringblock->lcd_string.selector_0_string.data, bytes_to_copy);
		} else {
			int string_offset;

			bytes_to_copy = MIN(lcdstring_len - bytes_copied, IPMI_DELL_LCD_STRINGN_SIZE);
			if (bytes_to_copy < 1)
				break;
			string_offset = IPMI_DELL_LCD_STRING1_SIZE + IPMI_DELL_LCD_STRINGN_SIZE * (ii-1);
			memcpy (lcdstring+string_offset, lcdstringblock->lcd_string.selector_n_data, bytes_to_copy);
		}

		bytes_copied += bytes_to_copy;
		if (bytes_copied >= lcdstring_len)
			break;
	}
	return 0;
}

static int
ipmi_lcd_get_info(struct ipmi_intf * intf)

{
	struct ipmi_rs * rsp = NULL;
	struct ipmi_rq req = {0};
	uint8_t data[4];
	IPMI_DELL_LCD_CAPS * lcd_caps;
	uint8_t command = 0;

	printf("LCD info\n");

	if (ipmi_lcd_get_configure_command (intf, &command) != 0) {
		return -1;
	} else {
		if (command == IPMI_DELL_LCD_CONFIG_DEFAULT) {
			char text[IPMI_DELL_LCD_STRING_LENGTH_MAX+1] = {0};

			ipmi_lcd_get_platform_model_name(intf, text, IPMI_DELL_LCD_STRING_LENGTH_MAX,
													IPMI_DELL_PLATFORM_MODEL_NAME_SELECTOR);

			if (text == NULL)
				return -1;
			printf("    Setting: default\n");
			printf("    Line 1:  %s\n", text);
		} else if (command == IPMI_DELL_LCD_CONFIG_NONE) {
			printf("    Setting:   none\n");
		} else if (command == IPMI_DELL_LCD_CONFIG_USER_DEFINED) {
			req.msg.netfn = IPMI_NETFN_APP;
			req.msg.cmd = IPMI_GET_SYS_INFO;
			req.msg.data_len = 4;
			req.msg.data = data;
			data[0] = 0;				// get parameter
			data[1] = IPMI_DELL_LCD_GET_CAPS_SELECTOR;
			data[2] = 0;				// set selector (n/a)
			data[3] = 0;				// block selector (n/a)

			printf("    Setting: custom\n");

			rsp = intf->sendrecv(intf, &req);
			if (rsp == NULL) {
				lprintf(LOG_ERR, " Error getting LCD capabilities.");
				return -1;
			} else if (rsp->ccode > 0) {
				lprintf(LOG_ERR, " Error getting LCD capabilities: %s",
					val2str(rsp->ccode, completion_code_vals));
				return -1;
			}

			lcd_caps = (IPMI_DELL_LCD_CAPS *)(void *)rsp->data;
			if (lcd_caps->number_lines > 0) {
				char *text;
				char lcdstring[IPMI_DELL_LCD_STRING_LENGTH_MAX+1] = {0};
				text = lcdstring;
				int rc;

				rc = ipmi_lcd_get_single_line_text (intf, text, lcd_caps->max_chars[0]);
				if (text == NULL)
					return -1;
				printf("    Text:    %s\n", text);
				//printf("    Length:  %d\n", strlen(text));
			} else {
				printf("    No lines to show\n");
			}
		}
	}

	return 0;
}

static int 
ipmi_lcd_set_single_line_text (struct ipmi_intf * intf, char * text)
{
	struct ipmi_rs * rsp = NULL;
	struct ipmi_rq req = {0};
	uint8_t data[18];
	int bytes_to_store = strlen(text);
	int bytes_stored = 0;
	int ii;
	int rc = 0;

	bytes_to_store = MIN(bytes_to_store, IPMI_DELL_LCD_STRING_LENGTH_MAX);
	for (ii = 0; ii < 4; ii++) {
		// first block, 2 bytes parms and 14 bytes data
		if (0 == ii) {
			int size_of_copy =  
				MIN((bytes_to_store - bytes_stored), IPMI_DELL_LCD_STRING1_SIZE);
			if (size_of_copy < 0)		// allow 0 string length
				break;
			req.msg.netfn = IPMI_NETFN_APP;
			req.msg.cmd = IPMI_SET_SYS_INFO;
			req.msg.data_len = size_of_copy + 4; // chars, selectors and sizes
			req.msg.data = data;
			data[0] = IPMI_DELL_LCD_STRING_SELECTOR;
			data[1] = ii;				// block number to use (0)
			data[2] = 0;				// string encoding
			data[3] = bytes_to_store;	// total string length
			memcpy (data+4, text+bytes_stored, size_of_copy);
			bytes_stored += size_of_copy;
		} else {
			int size_of_copy =  
				MIN((bytes_to_store - bytes_stored), IPMI_DELL_LCD_STRINGN_SIZE);
			if (size_of_copy <= 0)
				break;
			req.msg.netfn = IPMI_NETFN_APP;
			req.msg.cmd = IPMI_SET_SYS_INFO;
			req.msg.data_len = size_of_copy + 2;
			req.msg.data = data;
			data[0] = IPMI_DELL_LCD_STRING_SELECTOR;
			data[1] = ii;				// block number to use (1,2,3)
			memcpy (data+2, text+bytes_stored, size_of_copy);
			bytes_stored += size_of_copy;
		}

		rsp = intf->sendrecv(intf, &req);
		if (rsp == NULL) {
			lprintf(LOG_ERR, " Error setting text data");
			rc = -1;
		} else if (rsp->ccode > 0) {
			lprintf(LOG_ERR, " Error setting text data: %s",
				val2str(rsp->ccode, completion_code_vals));
			rc = -1;
		}
	}

	return rc;
}

static int
ipmi_lcd_set_text(struct ipmi_intf * intf, char * text, int line_number)
{
	int rc = 0;

	struct ipmi_rs * rsp = NULL;
	struct ipmi_rq req = {0};
	uint8_t data[4];
	IPMI_DELL_LCD_CAPS * lcd_caps;

	req.msg.netfn = IPMI_NETFN_APP;
	req.msg.cmd = IPMI_GET_SYS_INFO;
	req.msg.data_len = 4;
	req.msg.data = data;
	data[0] = 0;				// get parameter
	data[1] = IPMI_DELL_LCD_GET_CAPS_SELECTOR;
	data[2] = 0;				// set selector (n/a)
	data[3] = 0;				// block selector (n/a)

	rsp = intf->sendrecv(intf, &req);
	if (rsp == NULL) 
	{
		lprintf(LOG_ERR, " Error getting LCD capabilities");
		return -1;
	}
	else if (rsp->ccode > 0) 
	{
		lprintf(LOG_ERR, " Error getting LCD capabilities: %s",
			val2str(rsp->ccode, completion_code_vals));

		return -1;
	}

	lcd_caps = (IPMI_DELL_LCD_CAPS *)(void *)rsp->data;

	if (lcd_caps->number_lines > 0) {
		rc = ipmi_lcd_set_single_line_text (intf, text);
	} else {
		lprintf(LOG_ERR, "LCD does not have any lines that can be set");
		rc = -1;
	}


	return rc;
}

int
ipmi_lcd_configure (struct ipmi_intf * intf, int command, int8_t line_number, char * text)
{
	int rc = 0;


	if (IPMI_DELL_LCD_CONFIG_USER_DEFINED == command)
		// Any error was reported earlier. 
		rc = ipmi_lcd_set_text(intf, text, line_number);

	if (rc == 0)
		rc = ipmi_lcd_set_configure_command (intf, command);

	return rc;
}


#if 0
xxxxxxxxxxxxx
static int 
ipmi_read_dell_fru(struct ipmi_intf * intf, int8_t * fru_buffer, int fru_size_max)
{
	int rc = 0;
	struct ipmi_rs * rsp = NULL;
	struct ipmi_rq req = {0};
	uint8_t data[2];


	memset(&req, 0, sizeof(req));
	req.msg.netfn = IPMI_NETFN_STORAGE;
	req.msg.cmd = GET_FRU_INFO;
	req.msg.data = msg_data;
	req.msg.data_len = 1;
	req.msg.data[0] = 0;				// 

	rsp = intf->sendrecv(intf, &req);
	if (rsp == NULL) {
		printf(" Device not present (No Response)\n");
		return -1;
	}
	if (rsp->ccode > 0) {
		printf(" Device not present (%s)\n",
			val2str(rsp->ccode, completion_code_vals));
		return -1;
	}





	memset(&req, 0, sizeof(req));
	req.msg.netfn = IPMI_NETFN_STORAGE;
	req.msg.cmd = GET_FRU_DATA;
	req.msg.data = msg_data;
	req.msg.data_len = 4;

	rsp = intf->sendrecv(intf, &req);
	if (rsp == NULL) 
	{
		lprintf(LOG_ERR, " Error getting LCD capabilities");
		return -1;
	}
	else if (rsp->ccode > 0) 
	{
		lprintf(LOG_ERR, " Error getting LCD capabilities: %s",
			val2str(rsp->ccode, completion_code_vals));

		return -1;
	}
}
#endif

static int 
ipmi_sysinfo_id(struct ipmi_intf * intf, int type)
{
	struct ipmi_rs * rsp = NULL;
	struct ipmi_rq req = {0};
	int rc = 0;
	struct ipm_devid_rsp * dev_id;
	struct sdr_repo_info_rs * repo_info;
	uint8_t ipmi_version;
																// ipmi get system info APP fn=06 cmd=01
	// 20 80 01 52 51 9f a2 02 00 00 00 00 00 00 00 <- raw data
	memset(&req, 0, sizeof(req));
	req.msg.netfn = IPMI_NETFN_APP;
	req.msg.cmd = BMC_GET_DEVICE_ID;
	req.msg.data = NULL;
	req.msg.data_len = 0;

	rsp = intf->sendrecv(intf, &req);
	if (rsp == NULL) {
		printf(" Error getting device ID (No Response)\n");
		return -1;
	}
	if (rsp->ccode > 0) {
		printf(" Error getting device ID (%s)\n",
			val2str(rsp->ccode, completion_code_vals));
		return -1;
	}

	dev_id = (struct ipm_devid_rsp *)(void *)rsp->data;

	printf("Device ID           : %0.2d\n", dev_id->device_id);
	printf("Device Revision     : %0.2d\n", dev_id->device_revision & 0x7F);
	printf("Firmware Version    : %d.%0.2x\n", dev_id->fw_rev1 & 0x7F, 
		dev_id->fw_rev2); // BCD value. byte->hex conversion used.
	printf("IPMI Version        : %d.%d\n", dev_id->ipmi_version & 0x0F, dev_id->ipmi_version >> 4);
	ipmi_version = dev_id->ipmi_version;

//	printf("test ipmiversion = %x\n", dev_id->ipmi_version);

	printf("Manufacturer ID     : %lu\n", 
		(uint32_t)(dev_id->manufacturer_id[2] & 0x0f) * 65536 + 
		(uint32_t)(dev_id->manufacturer_id[1] * 256) + 
		(uint32_t)dev_id->manufacturer_id[0]);
	printf("Product ID          : %d\n", dev_id->product_id[1]*256 + dev_id->product_id[0]);

	memset(&req, 0, sizeof(req));
	req.msg.netfn = IPMI_NETFN_APP;
	req.msg.cmd = BMC_GET_SELF_TEST;
	req.msg.data = NULL;
	req.msg.data_len = 0;
	rsp = intf->sendrecv(intf, &req);
//	printf("Status              : No error;\n",-1);

//	printf("errors: byte 2: %02X, byte 3: %02X, ccode/byte1: %08X\n\n\n", rsp->data[0], rsp->data[1],rsp->data);
//	rsp->data[0] = 0x57;
//	rsp->data[1] = 0x00;
//	printf("changed error: %02X\n",rsp->data[0]);

	if (rsp->data[0] == 0x55) { printf("Status              : No error\n",-1); }
	else if (rsp->data[0] == 0x56) { printf("Status              : Self Test Function not implemented in this controller\n"); }
	else if (rsp->data[0] == 0x58) { printf("Status              : Fatal Hardware Error (BMC is inoperative)\n"); }
	else if (rsp->data[0] == 0x57) { 	
		printf("Status              : ");
		if (rsp->data[1] & 0x01) {printf("Controller operational firmware corrupte "); }
		if (rsp->data[1] & 0x02) {printf("Controller update 'boot block' firmware corrupte "); }
		if (rsp->data[1] & 0x04) {printf("Internal Use Area of BMC FRU corrupted "); }
		if (rsp->data[1] & 0x08) {printf("SDR Repository Empty "); }
		if (rsp->data[1] & 0x10) {printf("IPMB signal lines do not respond "); }
		if (rsp->data[1] & 0x20) {printf("Cannot access BMC FRU device "); }
		if (rsp->data[1] & 0x40) {printf("Cannot access SDR Repository "); }
		if (rsp->data[1] & 0x80) {printf("Cannot access SEL device"); }
		printf("\n");
	}
	else if (rsp->data[0] != 0xFF) { printf("Status              : Internal Failure. Refer to particular device's specification for definition\n"); }
	else { printf("Status              : \n"); }

	memset(&req, 0, sizeof(req));
	req.msg.netfn = IPMI_NETFN_STORAGE;
	req.msg.cmd = GET_SDR_REPO_INFO;
	req.msg.data = NULL;
	req.msg.data_len = 0;

	rsp = intf->sendrecv(intf, &req);
	if (rsp == NULL) {
		printf(" Error getting SDR version (No Response)\n");
		return -1;
	}
	if (rsp->ccode > 0) {
		printf(" Error getting SDR version (%s)\n",
			val2str(rsp->ccode, completion_code_vals));
		return -1;
	}
	
	repo_info = (struct sdr_repo_info_rs *)(void *)rsp->data;
	printf("SDR Version         : %d.%d\n", repo_info->version & 0x0F, repo_info->version >> 4);


	memset(&req, 0, sizeof(req));
	req.msg.netfn = IPMI_NETFN_APP;
	req.msg.cmd = BMC_GET_GUID;
	req.msg.data = NULL;
	req.msg.data_len = 0;

	rsp = intf->sendrecv(intf, &req);
	if (rsp == NULL) {
		printf(" Error getting device GUID (No Response)\n");
		return -1;
	}
	if (rsp->ccode > 0) {
		printf(" Error getting device GUID (%s)\n",
			val2str(rsp->ccode, completion_code_vals));
		return -1;
	}
	printf("GUID                : %.2X%.2X%.2X%.2X-%.2X%.2X-%.2X%.2X-%.2X%.2X-%.2X%.2X%.2X%.2X%.2X%.2X\n",
		rsp->data[15], rsp->data[14], rsp->data[13], rsp->data[12],
		rsp->data[11], rsp->data[10], rsp->data[9], rsp->data[8], 
		rsp->data[7], rsp->data[6], rsp->data[5], rsp->data[4], 
		rsp->data[3], rsp->data[2], rsp->data[1], rsp->data[0]);


	rc = ipmi_sysinfo_fru(intf, 0, ipmi_version);
	return rc;
}

/* __ipmi_fru_print  -  Do actual work to print a FRU by its ID
 *
 * @intf:	ipmi interface
 * @id:		fru id
 *
 * returns -1 on error
 * returns 0 if successful
 * returns 1 if device not present
 */
static int
ipmi_sysinfo_fru(struct ipmi_intf * intf, uint8_t id, uint8_t ipmi_version)
{
	struct ipmi_rs * rsp;
	struct ipmi_rq req;
	struct fru_info fru;
	struct fru_header header;
	uint8_t msg_data[4];

	

	memset(&fru, 0, sizeof(struct fru_info));
	memset(&header, 0, sizeof(struct fru_header));

	/*
	 * get info about this FRU
	 */
	memset(msg_data, 0, 4);
	msg_data[0] = id;

	memset(&req, 0, sizeof(req));
	req.msg.netfn = IPMI_NETFN_STORAGE;
	req.msg.cmd = GET_FRU_INFO;
	req.msg.data = msg_data;
	req.msg.data_len = 1;

	rsp = intf->sendrecv(intf, &req);
	if (rsp == NULL) {
		printf(" Device not present (No Response)\n");
		return -1;
	}
	if (rsp->ccode > 0) {
		printf(" Device not present (%s)\n",
			val2str(rsp->ccode, completion_code_vals));
		return -1;
	}

	fru.size = (rsp->data[1] << 8) | rsp->data[0];
	fru.access = rsp->data[2] & 0x1;

	lprintf(LOG_DEBUG, "fru.size = %d bytes (accessed by %s)",
		fru.size, fru.access ? "words" : "bytes");

	if (fru.size < 1) {
		lprintf(LOG_ERR, " Invalid FRU size %d", fru.size);
		return -1;
	}

	/*
	 * retrieve the FRU header
	 */
	msg_data[0] = id;
	msg_data[1] = 0;
	msg_data[2] = 0;
	msg_data[3] = 8;

	memset(&req, 0, sizeof(req));
	req.msg.netfn = IPMI_NETFN_STORAGE;
	req.msg.cmd = GET_FRU_DATA;
	req.msg.data = msg_data;
	req.msg.data_len = 4;

	rsp = intf->sendrecv(intf, &req);
	if (rsp == NULL) {
		printf(" Device not present (No Response)\n");
		return 1;
	}
	if (rsp->ccode > 0) {
		printf(" Device not present (%s)\n",
		       val2str(rsp->ccode, completion_code_vals));
		return 1;
	}

	if (verbose > 1)
		printbuf(rsp->data, rsp->data_len, "FRU DATA");

	memcpy(&header, rsp->data + 1, 8);

	if (header.version != 1) {
		lprintf(LOG_ERR, " Unknown FRU header version 0x%02x",
			header.version);
		return -1;
	}

	/* offsets need converted to bytes
	 * but that conversion is not done to the structure
	 * because we may end up with offset > 255
	 * which would overflow our 1-byte offset field */



	char assettag[IPMI_DELL_SYSINFO_ASSET_TAG_LENGTH];
	char servicetag[IPMI_DELL_SYSINFO_SERVICE_TAG_LENGTH] = {0};
	char hostname[IPMI_DELL_LCD_STRING_LENGTH_MAX+1] = {0};
	char productModel[IPMI_DELL_LCD_STRING_LENGTH_MAX+1] = {0};
	char osName[IPMI_DELL_LCD_STRING_LENGTH_MAX+1] = {0};
	char biosversion[IPMI_DELL_LCD_STRING_LENGTH_MAX+1] = {0};

	/* Print board area */
	if ((header.offset.board*8) >= sizeof(struct fru_header))
		ipmi_sysinfo_print_board(intf, &fru, id, header.offset.board*8, ipmi_version);
//printf("offset = %x\n", header.offset.internal*8);
	/* Print the rest of the sysinfo command */
	if (ipmi_version == 0x51) {
		
//		ipmi_sysinfo_print_ipmi15(intf, &fru, id, header.offset.internal*8);
	} else {

/*	memcpy(hostname, ipmi_lcd_get_platform_model_name(intf, IPMI_DELL_LCD_STRING_LENGTH_MAX, 
							IPMI_DELL_SYSINFO_HOST_NAME), IPMI_DELL_LCD_STRING_LENGTH_MAX);*/
		ipmi_lcd_get_platform_model_name(intf, hostname, IPMI_DELL_LCD_STRING_LENGTH_MAX, 
							IPMI_DELL_SYSINFO_HOST_NAME);
		printf("Host Name           : %s\n", hostname);


/*	memcpy(productModel, ipmi_lcd_get_platform_model_name(intf, IPMI_DELL_LCD_STRING_LENGTH_MAX, 
						  IPMI_DELL_PLATFORM_MODEL_NAME_SELECTOR), IPMI_DELL_LCD_STRING_LENGTH_MAX);*/
		ipmi_lcd_get_platform_model_name(intf, productModel, IPMI_DELL_LCD_STRING_LENGTH_MAX, 
						  IPMI_DELL_PLATFORM_MODEL_NAME_SELECTOR);
		printf("Product Model       : %s\n", productModel);

		ipmi_sysinfo_get_tag(intf, assettag, IPMI_DELL_SYSINFO_ASSET_TAG, 
										  IPMI_DELL_SYSINFO_ASSET_TAG_LENGTH);
		printf("Asset Tag           : %s\n", assettag);

//	printf("Test service tag = %s\n", servicetag);
		ipmi_sysinfo_get_tag(intf, servicetag, IPMI_DELL_SYSINFO_SERVICE_TAG, 
										  IPMI_DELL_SYSINFO_SERVICE_TAG_LENGTH);
		printf("Service Tag         : %s\n", servicetag);

		ipmi_lcd_get_platform_model_name(intf, biosversion, IPMI_DELL_LCD_STRING_LENGTH_MAX, 
											 IPMI_DELL_SYSINFO_BIOS_VERSION);
		printf("BIOS Version        : %s\n", biosversion);
		   
/*	memcpy (osName, ipmi_lcd_get_platform_model_name(intf, IPMI_DELL_LCD_STRING_LENGTH_MAX, 
											 IPMI_DELL_SYSINFO_OS_NAME), IPMI_DELL_LCD_STRING_LENGTH_MAX);*/
		ipmi_lcd_get_platform_model_name(intf, osName, IPMI_DELL_LCD_STRING_LENGTH_MAX, 
											 IPMI_DELL_SYSINFO_OS_NAME);
		printf("System OS Name      : %s\n", osName);

	}


	return 0;
}

/* fru_area_print_board  -  Print FRU Board Area
 *
 * @intf:	ipmi interface
 * @fru:	fru info
 * @id: 	fru id
 * @offset:	offset pointer
 */
static void
ipmi_sysinfo_print_board(struct ipmi_intf * intf, struct fru_info * fru,
		     uint8_t id, uint32_t offset, uint8_t ipmi_version)
{
	char * fru_area;
	char * bios_version; 
	char * prod_model;
	char * service_tag;
	char * asset_tag;
	uint8_t * fru_data;
	uint8_t language;
	uint32_t fru_len, area_len, i, j;
	uint16_t length;
	uint16_t bib_offset, bib_info_area, bib_length, bib_end;

	j = 0;

	i = offset;
	fru_len = 0;

//	printf("fru size = %x\n", fru->size);

	fru_data = malloc(fru->size + 1);
	if (fru_data == NULL) {
		lprintf(LOG_ERR, " Out of memory!");
		return;
	}
	memset(fru_data, 0, fru->size + 1);

	/* read enough to check length field */
	if (read_fru_area(intf, fru, id, i, 2, fru_data) == 0)
		fru_len = 8 * fru_data[i + 1];
	if (fru_len <= 0) {
		free(fru_data);
		return;
	}

	fru_len = 0x200;
	i = 0;
	/* read in the full fru */
	if (read_fru_area(intf, fru, id, i, fru_len, fru_data) < 0) {
		free(fru_data);
		return;
	}


	language = fru_data[10];
	if (language == 0x00) {
		printf("Board Language Code : English\n");
	} else {
		printf("Board Language Code : Unknown\n");
	}

/*	for (i=0; i<0x200; i++) {
		
		if (i%16 == 0) {
			printf("\n");
			printf("offset 0x%02x0 - ", j);
			j++;
		}
		if ((i+8)%16 == 0) {
			printf("| ");
		}
		printf("%02x ", fru_data[i]);
	}*/
	i = 8;
//	printf("i = %x\n", i);

	i++;	/* skip fru area version */
	area_len = fru_data[i++] * 8; /* fru area length */
	i++;	/* skip fru board language */
	i += 3;	/* skip mfg. date time */


	fru_area = get_fru_area_str(fru_data, &i);
	if (fru_area != NULL && strlen(fru_area) > 0) {
//		printf(" Board Mfg             : %s\n", fru_area);
		free(fru_area);
	}

	fru_area = get_fru_area_str(fru_data, &i);
	if (fru_area != NULL && strlen(fru_area) > 0) {
		printf("Board Product Name  : %s\n", fru_area);
		free(fru_area);
	}

	fru_area = get_fru_area_str(fru_data, &i);
	if (fru_area != NULL && strlen(fru_area) > 0) {
		printf("Board Serial Number : %s\n", fru_area);
		free(fru_area);
	}

	fru_area = get_fru_area_str(fru_data, &i);
	if (fru_area != NULL && strlen(fru_area) > 0) {
		printf("Board Part Number   : %s\n", fru_area);
		free(fru_area);
	}

	fru_area = get_fru_area_str(fru_data, &i);
	if (fru_area != NULL && strlen(fru_area) > 0) {
//		if (verbose > 0)
			printf("Board FRU File ID   : %s\n", fru_area);
		free(fru_area);
	}

	/* read any extra fields */
	while ((fru_data[i] != 0xc1) && (i < offset + area_len))
	{
		int j = i;
		fru_area = get_fru_area_str(fru_data, &i);
		if (fru_area != NULL && strlen(fru_area) > 0) {
			printf(" Board Extra           : %s\n", fru_area);
			free(fru_area);
		}
		if (i == j)
			break;
	}

	if (ipmi_version == 0x51) {
 

		/* get host name */
		length = (fru_data[0x72] << 8) | (fru_data[0x71]);
		fru_area = malloc(length+1);
		memset(fru_area, 0, length+1);
		memcpy(fru_area, &fru_data[0x75], length);
		fru_area[length] = '\0';
		printf("Host Name           : %s\n", fru_area);
		free(fru_area);

		fru_area = ipmi_get_bib_info(fru_data, BIB_TYPE_PDRODUCT_MODEL);
		if (fru_area != NULL && strlen(fru_area) > 0) {
			printf("Product Model       : %s\n", fru_area);
			free(fru_area);
		}
		fru_area = ipmi_get_bib_info(fru_data, BIB_TYPE_ASSET_TAG);
		if (fru_area != NULL && strlen(fru_area) > 0) {
			printf("Asset Tag           : %s\n", fru_area);
			free(fru_area);
		}
		fru_area = ipmi_get_bib_info(fru_data, BIB_TYPE_SERVICE_TAG);
		if (fru_area != NULL && strlen(fru_area) > 0) {
			printf("Service Tag         : %s\n", fru_area);
			free(fru_area);
		}
		fru_area = ipmi_get_bib_info(fru_data, BIB_TYPE_BIOS_VERSION);
		if (fru_area != NULL && strlen(fru_area) > 0) {
			printf("BIOS version        : %s\n", fru_area);
			free(fru_area);
		}
    }

	/* get BIB fields */
/*	if (fru_data[0x66] == 0xc3) {
		bib_offset = (fru_data[0x68] << 8) | fru_data[0x67];
	} else if (fru_data[0x69] == 0xc3) {
		bib_offset = (fru_data[0x6B] << 8) | fru_data[0x6A];
	}

	printf("BIB offset = %x\n", bib_offset);

	bib_info_area = bib_offset + 6;
	bib_length = (fru_data[bib_info_area+2]<<8) | fru_data[bib_info_area+1];
	bib_end = bib_info_area + bib_length;

	/* print bios version
	offset = bib_info_area;
	while ((fru_data[offset] != 00) && (offset <= bib_end)) {
		offset += fru_data[offset+1]+3;
	}
	if (fru_data[offset] == 0) {	/* bios info found 
		length = fru_data[offset+1];
		bios_version = malloc(length+1);
		memset(bios_version, 0, length+1);
		memcpy(bios_version, &fru_data[offset+2], length);
		bios_version[length] = '\0';
		printf("BIOS version: %s\n", bios_version);
	}

	/* print product model 
	offset = bib_info_area;
/*	while ((fru_data[offset] != 0x1a) && (offset <= bib_end)) {
		printf("offset = %x\n", offset);
		offset += fru_data[offset+1]+3;
	}
	if (fru_data[offset] == 0x1a) {	/* product model found 
		length = fru_data[offset+1]-6;
//		printf("product model offset = %x\n",offset);
//		printf("product model length @@@@@@@@@@@@@@@= %x\n",length);
		prod_model = malloc(length+1);
		memset(prod_model, 0, length+1);
		memcpy(prod_model, &fru_data[offset+8], length);
		prod_model[length] = '\0';
/*		for (i=0; i<=length; i++) {
			printf("%02x ", prod_model[i]); 
		}
		printf("\n");
		printf("Product Model: %s\n", prod_model);
	}

		/* print service tag
	offset = bib_info_area;
	while ((fru_data[offset] != 0x01) && (offset <= bib_end)) {
		offset += fru_data[offset+1]+3;
	}
	if (fru_data[offset] == 0x01) {	/* service tag found 
		length = fru_data[offset+1];
		service_tag = malloc(length+1);
		memset(service_tag, 0, length+1);
		memcpy(service_tag, &fru_data[offset+2], length);
		service_tag[length] = '\0';
		printf("service tag: %s\n", service_tag);
	}

			/* print asset tag
	offset = bib_info_area;
	while ((fru_data[offset] != 0x02) && (offset <= bib_end)) {
		offset += fru_data[offset+1]+3;
	}
	if (fru_data[offset] == 0x02) {	/* asset tag found 
		length = fru_data[offset+1];
		asset_tag = malloc(length+1);
		memset(asset_tag, 0, length+1);
		memcpy(asset_tag, &fru_data[offset+2], length);
		asset_tag[length] = '\0';
		printf("Asset tag: %s\n", asset_tag);
	}




	/* get product model */
/*	length = (fru_data[0x139 + bib_offset];
	fru_area = malloc(length+1);
	memset(fru_area, 0, length+1);
	memcpy(fru_area, &fru_data[0x75], length);
	fru_area[length] = '\0';
	printf("Host Name: %s\n", fru_area);*/

	
/*	printf("Board Language Code : %s\n", boardinfo.lang);
	printf("Board Product Name  : %s\n", boardinfo.prod_name);
	printf("Board Serial Number : %s\n", boardinfo.serial);
	printf("Board Part Number   : %s\n", boardinfo.part);
	printf("Board FRU File ID   : %s\n", boardinfo.fru);*/

	free(fru_data);
}

static char*  ipmi_get_bib_info(uint8_t * fru_data, uint8_t bib_type)
{
	uint16_t bib_offset, bib_info_area, bib_length, bib_end, length;
	uint32_t offset;
	char * bib_data;

		/* get BIB fields */
	if (fru_data[0x66] == 0xc3) {
		bib_offset = (fru_data[0x68] << 8) | fru_data[0x67];
	} else if (fru_data[0x69] == 0xc3) {
		bib_offset = (fru_data[0x6B] << 8) | fru_data[0x6A];
	}

//	printf("BIB offset = %x\n", bib_offset);

	bib_info_area = bib_offset + 6;
	bib_length = (fru_data[bib_info_area+2]<<8) | fru_data[bib_info_area+1];
	bib_end = bib_info_area + bib_length;

	/* find BIB field */
	offset = bib_info_area;
	while ((fru_data[offset] != bib_type) && (offset <= bib_end)) {
		offset += fru_data[offset+1]+3;
	}
	if (fru_data[offset] == bib_type) {	/* bib info found */
		if (bib_type == BIB_TYPE_PDRODUCT_MODEL) {
			length = fru_data[offset+1]-6;
		} else {
			length = fru_data[offset+1];
		}
		
		bib_data = malloc(length+1);
		memset(bib_data, 0, length+1);
		if (bib_type == BIB_TYPE_PDRODUCT_MODEL) {
			memcpy(bib_data, &fru_data[offset+8], length);
		} else {
			memcpy(bib_data, &fru_data[offset+2], length);
		}
		bib_data[length] = '\0';
//		printf("BIOS version: %s\n", bib_data);
	} else {
		bib_data = NULL;
	}
	return bib_data;
}

static int ipmi_sysinfo_get_tag(struct ipmi_intf * intf, char* tagstring, uint8_t tag_type, uint8_t length)
{
	struct ipmi_rs * rsp = NULL;
	struct ipmi_rq req = {0};
	uint8_t data[4];
	IPMI_DELL_TAG * tagblock;
	int lcdstring_len = 0;
	int bytes_copied = 0;
//	char tagstring[11] = {0};
	int ii;

		int bytes_to_copy;

		req.msg.netfn = IPMI_NETFN_APP;
		req.msg.cmd = IPMI_GET_SYS_INFO;
		req.msg.data_len = 4;
		req.msg.data = data;
		data[0] = 0;				// get parameter
		data[1] = tag_type;/*to get asset tag*/
		data[2] = 0;
		data[3] = 0;

		rsp = intf->sendrecv(intf, &req);

		if (rsp == NULL) {
			lprintf(LOG_ERR, " Error getting asset tag");
			return -1;
		} else if (rsp->ccode > 0) {
			lprintf(LOG_ERR, " Error getting asset tag: %s",
				val2str(rsp->ccode, completion_code_vals));
			return -1;
		}

		tagblock = (IPMI_DELL_TAG *) (void *) rsp->data;
//		printf("tagblock string = %s\n", (char*)tagblock->tag);
//		printf("tag length = %d\n", tagblock->length);
//		printf("tagstring before copy = %s\n", tagstring);
		memset(tagstring, 0,11);
//		strcpy(tagstring, (char*)tagblock->tag);
		memcpy (tagstring, &tagblock->tag, length);
//		printf("tagstring after copy = %s\n", tagstring);

		return 0;

}



int ipmi_sel_print(struct ipmi_intf * intf, uint16_t count, int printOption)
{
	CSLFUSERAPI		pfnApiList;
	DECODING_METHOD	method;
	unsigned char	severity;
	uint8_t			index = 1;
	int				status = COMMON_STATUS_BAD_PARAMETER;
	unsigned short	TimeStrSize = LOG_DATE_STR_SIZE;
	unsigned short	DescriptionStrSize = EVENT_DESC_STR_SIZE;
	uint16_t		curr_id = 0;
	uint16_t		next_id;
	uint8_t			sel_entry[0xff];
	uint8_t			test;
	uint16_t		last = 0;
	uint16_t 		totalEntries = 0;
	uint16_t		start_id = 0;
	uint16_t		start = 1;
	int				rc;
	int				error_count;


	char LogTimeString[LOG_DATE_STR_SIZE];
	char DateString[LOG_DATE_STR_SIZE];
	char DescriptionString[EVENT_DESC_STR_SIZE];

	rc = ipmi_sdr_build_table(intf);
	if (rc == -1) {
		return rc;
	}

			struct ipmi_rs * rsp;
			struct ipmi_rq req;
			

			memset(&req, 0, sizeof(req));
			req.msg.netfn = IPMI_NETFN_STORAGE;
			req.msg.cmd = IPMI_CMD_GET_SEL_INFO;

			rsp = intf->sendrecv(intf, &req);
			if (rsp == NULL) {
				lprintf(LOG_ERR, " Error getting sel info");
				return -1;
			} else if (rsp->ccode > 0) {
				lprintf(LOG_ERR, " Error getting sel info: %s",
				val2str(rsp->ccode, completion_code_vals));
				return -1;
			}

			totalEntries = ((rsp->data[2] << 8) | rsp->data[1]);

	 	if (printOption == SEL_PRINT_FIRST) 
		{
			if(count > totalEntries)
				last = totalEntries;
			else
				last = count;
		}
	    if (printOption == SEL_PRINT_LAST)
		{
			if (count <=  totalEntries)
			{
				start_id = totalEntries - count;
				start = start_id+1;
		  	 	/*added here Meghna DF:170716*/
				index = start;
				last = totalEntries;
			}
		}
		if(printOption == SEL_PRINT_ALL)
		{
				last = totalEntries;
		}


	/* set decoding method */
	rc = ipmi_sel_sysinfo(intf, &method);
	if (rc == -1) {
		return -1;
	}
	
	status = CSLFSetDecodingMethod(&method);
	if (COMMON_STATUS_SUCCESS == status) {
		/* clear API structure */
		memset((void*) &pfnApiList, 0, sizeof(CSLFUSERAPI));
		pfnApiList.GetFirstSDR = sdr_get_first;
		pfnApiList.GetNextSDR = sdr_get_next;

		CSLFAttach(&pfnApiList);
		while (start <= last) {

			error_count = 0;
			do {
				next_id = ipmi_sel_get_record(intf, start, sel_entry);
				error_count ++;
				if (next_id == NULL) {
					printf("SDR is NULL and error_count is %d\n", error_count);
				}
			} while ( (next_id == NULL) && (error_count < 3) );
			
			if (next_id == NULL) {
				/*error message handled in ipmi_sel_get_record*/
				return -1;
			}
			start = next_id;
//			printf("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n");
		//	printf("Meghna:RecordId = %02x%02x\n", sel_entry[1], sel_entry[0]);
//			printf("Sensor# = %02x\n", sel_entry[11]);
		//	printf("Meghna:start = %d, index = %d, last = %d\n", start, index, last);
			
	//		if ((index >= start )&& (index <= last))
            {
			
				LogTimeString[0] = 0;
				DateString[0] = 0;
				DescriptionString[0] = 0;

//			printf("Line 915\n");

				status = CSLFSELEntryToStr(
				(void*)sel_entry,
				0,
				LogTimeString,
				&TimeStrSize,
				DescriptionString,
				&DescriptionStrSize,
				&severity,
				(void*)intf);


//			printf("Line 927\n");

			/*Convert date string*/
				status = CSLFSELUnixToCTime(LogTimeString, DateString);


//				printf("curr_id = %x\n", curr_id);

				printf("Severity          : %s\n", g_StatusTable[severity]);
				printf("Date and Time     : %s\n", DateString);
				printf("Description       : %s\n\n", DescriptionString);
				
			}
		//	index++;
		}
		CSLFDetach();
	}
	return 0;
}

static int make_int(const char *str, int *value)
{
	char *tmp=NULL;
	*value = strtol(str,&tmp,0);
	if ( tmp-str != strlen(str) )
	{
		return -1;
	}
	return 0;
}

extern int  ipmi_sel_get(struct ipmi_intf * intf, uint16_t id)
{
	uint16_t curr_id, next_id;
	SDRType * sdr;
	int rc;

	uint8_t *sdr_out;

	curr_id = 0x0000;
	next_id = 0x0000;
	while (curr_id != 0xFFFF) {
//		next_id = ipmi_sel_get_record(intf, curr_id);
		curr_id = next_id;
	}

	uint16_t reserveId;

//	rc = ipmi_sdr_reserve(intf, &reserveId);
//	printf("Reserve ID = %04x\n", reserveId);

//	sdr = ipmi_sdr_get(intf,reserveId,0);
//	sdr = ipmi_sdr_get(intf, reserveId, 0x0001);

//	sdr = ipmi_sdr_get_first((void*)intf);

//	sdr_out = (uint8_t*)sdr;
//	printf("@@@@@@@@@Current ID out = %02x%02x\n", sdr_out[1], sdr_out[0]);
//	sdr = ipmi_sdr_get_next(sdr, (void*)intf);
//	sdr = ipmi_sdr_get_next(sdr, (void*)intf);

	DECODING_METHOD method;
	rc = ipmi_sel_sysinfo(intf, &method);
	if (rc == -1) {
		return -1;
	}
	printf("CompletionCode = %d\n", method.completionCode);
	printf("deviceId = %d\n", method.deviceID);
	printf("deviceRevision = %d\n", method.deviceRevision);
	printf("firmwareMajor = %d\n", method.firmwareMajor);
	printf("firmwareMinor = %d\n", method.firmwareMinor);
	printf("ipmiVersion = %d\n", method.ipmiVersion);
}

static int
ipmi_sel_get_info(struct ipmi_intf * intf)
{
	struct ipmi_rs * rsp;
	struct ipmi_rq req;
	uint16_t e, f, version;
	int pctfull = 0;
	uint32_t fs    = 0xffffffff;
	uint32_t zeros = 0;


	memset(&req, 0, sizeof(req));
	req.msg.netfn = IPMI_NETFN_STORAGE;
	req.msg.cmd = IPMI_CMD_GET_SEL_INFO;

	rsp = intf->sendrecv(intf, &req);
	if (rsp == NULL) {
		lprintf(LOG_ERR, "Get SEL Info command failed");
		return -1;
	}
	if (rsp->ccode > 0) {
		lprintf(LOG_ERR, "Get SEL Info command failed: %s",
		       val2str(rsp->ccode, completion_code_vals));
		return -1;
	}
	if (verbose > 2)
		printbuf(rsp->data, rsp->data_len, "sel_info");

	printf("SEL Information\n");
        version = rsp->data[0];
	printf("Version          : %d.%d (%s)\n",
	       version & 0xf, (version>>4) & 0xf,
	       (version == 0x51 || version == 0x02) ? "v1.5, v2 compliant" : "Unknown");

	/* save the entry count and free space to determine percent full */
	e = buf2short(rsp->data + 1);
	f = buf2short(rsp->data + 3);
	printf("Entries          : %d\n", e);
	printf("Free Space       : %d bytes\n", f);
	if (e) {
		e *= 16;
		f += e;
		pctfull = (int)(100 * ( (double)e / (double)f ));
	}
	printf("Percent Used     : %d%%\n", pctfull);


	if ((!memcmp(rsp->data + 5, &fs,    4)) ||
		(!memcmp(rsp->data + 5, &zeros, 4)))
		printf("Last Add Time    : Not Available\n");
	else
		printf("Last Add Time    : %s\n",
			   ipmi_sel_timestamp(buf2long(rsp->data + 5)));

	if ((!memcmp(rsp->data + 9, &fs,    4)) ||
		(!memcmp(rsp->data + 9, &zeros, 4)))
		printf("Last Del Time    : Not Available\n");
	else
		printf("Last Del Time    : %s\n",
			   ipmi_sel_timestamp(buf2long(rsp->data + 9)));


	printf("Overflow         : %s\n",
	       rsp->data[13] & 0x80 ? "true" : "false");
	printf("Supported Cmds   : ");
        if (rsp->data[13] & 0x0f)
        {
	        if (rsp->data[13] & 0x08)
                        printf("'Delete' ");
	        if (rsp->data[13] & 0x04)
                        printf("'Partial Add' ");
	        if (rsp->data[13] & 0x02)
                        printf("'Reserve' ");
	        if (rsp->data[13] & 0x01)
                        printf("'Get Alloc Info' ");
        }
        else
                printf("None");
        printf("\n");

	/* get sel allocation info if supported */
	if (rsp->data[13] & 1) {
		memset(&req, 0, sizeof(req));
		req.msg.netfn = IPMI_NETFN_STORAGE;
		req.msg.cmd = IPMI_CMD_GET_SEL_ALLOC_INFO;

		rsp = intf->sendrecv(intf, &req);
		if (rsp == NULL) {
			lprintf(LOG_ERR,
				"Get SEL Allocation Info command failed");
			return -1;
		}
		if (rsp->ccode > 0) {
			lprintf(LOG_ERR,
				"Get SEL Allocation Info command failed: %s",
				val2str(rsp->ccode, completion_code_vals));
			return -1;
		}

		printf("# of Alloc Units : %d\n", buf2short(rsp->data));
		printf("Alloc Unit Size  : %d\n", buf2short(rsp->data + 2));
		printf("# Free Units     : %d\n", buf2short(rsp->data + 4));
		printf("Largest Free Blk : %d\n", buf2short(rsp->data + 6));
		printf("Max Record Size  : %d\n", rsp->data[7]);
	}
	return 0;
}

static char *
ipmi_sel_timestamp(uint32_t stamp)
{
	static char tbuf[40];
	time_t s = (time_t)stamp;
	memset(tbuf, 0, 40);
	strftime(tbuf, sizeof(tbuf), "%m/%d/%Y %H:%M:%S", gmtime(&s));				// edited here
	return tbuf;
}

uint16_t ipmi_sel_get_record(struct ipmi_intf * intf, uint16_t curr_id, uint8_t* sel_entry)
{
	struct ipmi_rq req;
	struct ipmi_rs * rsp;
	uint8_t msg_data[6];
	uint16_t next;
	int data_count;

	memset(msg_data, 0, 6);
	msg_data[0] = 0x00;	/* no reserve id, not partial get */
	msg_data[1] = 0x00;
	msg_data[2] = curr_id & 0xff;
	msg_data[3] = (curr_id >> 8) & 0xff;
	msg_data[4] = 0x00;	/* offset */
	msg_data[5] = 0xff;	/* length */

	memset(&req, 0, sizeof(req));
	req.msg.netfn = IPMI_NETFN_STORAGE;
	req.msg.cmd = IPMI_CMD_GET_SEL_ENTRY;
	req.msg.data = msg_data;
	req.msg.data_len = 6;

	rsp = intf->sendrecv(intf, &req);
	if (rsp == NULL) {
			lprintf(LOG_ERR, " Error getting sel record");
			return NULL;
	} else if (rsp->ccode > 0) {
			lprintf(LOG_ERR, " Error getting sel record: %s",
				val2str(rsp->ccode, completion_code_vals));
			return NULL;
	}

	/* save next entry id */
	next = (rsp->data[1] << 8) | rsp->data[0];
//	printf("next = %x\n", next);

//	printf("RecordID = %02x%02x\n", rsp->data[3], rsp->data[2]);

//	printf("Sensor # = %X\n", rsp->data[13]);

	memcpy((void*)sel_entry, (void*)&rsp->data[2], 16);





	return next;
}

int ipmi_sel_sysinfo(struct ipmi_intf *intf, DECODING_METHOD *method)
{
	struct ipmi_rs *rsp;
	struct ipmi_rq req;
	uint8_t data_in[16];
	struct ipm_devid_rsp * dev_id;

	memset(&req, 0, sizeof (req));

	req.msg.netfn = IPMI_NETFN_APP;
	req.msg.cmd = BMC_GET_DEVICE_ID;
	req.msg.data = NULL;
	req.msg.data_len = 0;

	rsp = intf->sendrecv(intf, &req);
	if (rsp == NULL) {
			lprintf(LOG_ERR, " Error getting system info");
			return -1;
		} else if (rsp->ccode > 0) {
			lprintf(LOG_ERR, " Error getting system info: %s",
				val2str(rsp->ccode, completion_code_vals));
			return -1;
		}

	method->completionCode = rsp->ccode;
	method->deviceID		= rsp->data[0];
	method->deviceRevision	= rsp->data[1];
	method->firmwareMajor	= rsp->data[2];
	method->firmwareMinor	= rsp->data[3];
	method->ipmiVersion		= rsp->data[4];
	method->supportFlags	= rsp->data[5];
	method->manufactureID[0]= rsp->data[6];
	method->manufactureID[1]= rsp->data[7];
	method->manufactureID[2]= rsp->data[8];
	method->productID[0]	= rsp->data[9];
	method->productID[1]	= rsp->data[10];
	method->auxiliary[0]	= rsp->data[11];
	method->auxiliary[1]	= rsp->data[12];
	method->auxiliary[2]	= rsp->data[13];
	method->auxiliary[3]	= rsp->data[14];

	return 1;

//	memcpy(method, &rsp->ccode, 1);
//	memcpy(method+1, &rsp->data, (sizeof(DECODING_METHOD)-1);
//	printf("decoding method size is %x\n", sizeof(DECODING_METHOD));
//	printf("Response code = %x\n", (uint8_t)method->completionCode);

}

/* ipmi_sdr_get_reservation  -  Obtain SDR reservation ID
 *
 * @intf:	ipmi interface
 * @reserve_id:	pointer to short int for storing the id
 *
 * returns 0 on success
 * returns -1 on error
 */
int
ipmi_sdr_reserve(struct ipmi_intf *intf, uint16_t * reserve_id)
{
	struct ipmi_rs *rsp;
	struct ipmi_rq req;

	/* obtain reservation ID */
	memset(&req, 0, sizeof (req));

//	if (use_built_in == 0) {
		req.msg.netfn = IPMI_NETFN_STORAGE;
//	} else {
//		req.msg.netfn = IPMI_NETFN_SE;
//	}

	req.msg.cmd = GET_SDR_RESERVE_REPO;
	rsp = intf->sendrecv(intf, &req);
//	printf("test2\n");
	/* be slient for errors, they are handled by calling function */
	if (rsp == NULL)
		return -1;
	if (rsp->ccode > 0)
		return -1;


	*reserve_id = ((struct sdr_reserve_repo_rs *) &(rsp->data))->reserve_id;
	lprintf(LOG_DEBUG, "SDR reservation ID %04x", *reserve_id);

	return 0;
}

static int
ipmi_sel_get_time(struct ipmi_intf * intf)
{
	struct ipmi_rs * rsp;
	struct ipmi_rq req;
	static char tbuf[40];
	uint32_t timei;
	time_t time;

	memset(&req, 0, sizeof(req));
	req.msg.netfn = IPMI_NETFN_STORAGE;
	req.msg.cmd   = IPMI_GET_SEL_TIME;

	rsp = intf->sendrecv(intf, &req);

	if (rsp == NULL) {
		lprintf(LOG_ERR, "Get SEL Time command failed");
		return -1;
	}
	if (rsp->ccode > 0) {
		lprintf(LOG_ERR, "Get SEL Time command failed: %s",
			val2str(rsp->ccode, completion_code_vals));
		return -1;
	}
	if (rsp->data_len != 4) {
		lprintf(LOG_ERR, "Get SEL Time command failed: "
			"Invalid data length %d", rsp->data_len);
		return -1;
	}

	memcpy(&timei, rsp->data, 4);
#if WORDS_BIGENDIAN
	time = (time_t)(BSWAP_32(timei));
#else
	time = (time_t)timei;
#endif

	strftime(tbuf, sizeof(tbuf), "%m/%d/%Y %H:%M:%S", gmtime(&time));
	printf("%s\n", tbuf);

	return 0;
}

static int
ipmi_sel_set_time(struct ipmi_intf * intf, const char * time_string)
{
	struct ipmi_rs     * rsp;
	struct ipmi_rq       req;
	struct tm            tm;
	time_t               t;
	uint32_t	     timei;
	const char *         time_format = "%m/%d/%Y %H:%M:%S";

	memset(&req, 0, sizeof(req));
	req.msg.netfn    = IPMI_NETFN_STORAGE;
	req.msg.cmd      = IPMI_SET_SEL_TIME;

	/* See if user requested set to current client system time */
	if (strncasecmp(time_string, "now", 3) == 0) {
		t = time(NULL);
	}
	else {
		/* Now how do we get our time_t from our ascii version? */
		if (strptime(time_string, time_format, &tm) == 0) {
			lprintf(LOG_ERR, "Specified time could not be parsed");
			return -1;
		}
		t = timegm(&tm);
		if (t < 0) {
			lprintf(LOG_ERR, "Specified time could not be parsed");
			return -1;
		}
	}

	timei = (uint32_t)t;
	req.msg.data = (uint8_t *)&timei;
	req.msg.data_len = 4;

#if WORDS_BIGENDIAN
	timei = BSWAP_32(timei);
#endif

	rsp = intf->sendrecv(intf, &req);
	if (rsp == NULL) {
		lprintf(LOG_ERR, "Set SEL Time command failed");
		return -1;
	}
	if (rsp->ccode > 0) {
		lprintf(LOG_ERR, "Set SEL Time command failed: %s",
			val2str(rsp->ccode, completion_code_vals));
		return -1;
	}

	ipmi_sel_get_time(intf);

	return 0;
}





/*--------------------------------------------------------------------*/
/**********************************************************************/

static int ipmi_sdr_build_table(struct ipmi_intf *intf)
{
	SDRType* sdr_cur;
	SDRType* sdr_next;
	IPMISDR current;
	IPMISDR next;
	IPMISDR sdr_out;
	sdr_cur = &current;
	sdr_next = &next;
	int rc;
	int i = 0;
	int j;
	uint8_t* test;
	int count;

	count = 0;
	do {
		rc = ipmi_build_sdr_first(intf, SDR[0].data);
		count ++;
//		printf("build_first rc = %x, count = %d\n", rc, count);
	} while ( (rc == -1) && (count < 3) );
	if (rc == -1) {
		printf("Cannot access SDR info:rc %x\n", rc);
		return rc;
	}
/*printf("\n");
	for (j=0;j<6;j++) {
		printf("data %d = %x\n", i+1, SDR[0].data[j]);
	}*/

	

	i = 0;

	do {

//		printf("\n");
//		printf("Getting data for entry %d\n", i+1);
		count = 0;
		do {
			rc = ipmi_build_sdr_next(intf, SDR[i].data, SDR[i+1].data);
			count++;
//			printf("build_next rc = %x, count = %d\n", rc, count);
		} while ( (rc == -1) && (count < 3) );
		 if (rc == -1) {
//			 printf("build_sdr_next returned %x\n", rc);
			 return rc;
		 }
		 
//		 for (j=0;j<6;j++) {
//			 printf("data %d = %x\n", i+1, SDR[i+1].data[j]);
//		 }

		 i++;
	}while ((rc != 1) && (i<MAX_SDR_ENTRIES));

	if (i >= MAX_SDR_ENTRIES) {
		printf("Error: SDR number limit exceeded.\n");
		return -1;
	}

	totalSDRCount = i;
//	printf("totalSDRCount = %d\n", totalSDRCount);


/*	if (rc == -1) {
		printf("Error getting SDR records\n");
		return rc;
	} else {
		return 0;
	}*/
}

static int ipmi_build_sdr_first(struct ipmi_intf *intf, uint8_t *data)
{
 //	printf("Entering get_first\n");
	uint16_t reserve_id;
	int rc;
	SDRType* sdr_out;
//	uint8_t  data[64];
	uint8_t* test;

	rc = ipmi_sdr_reserve(intf, &reserve_id);
	if (rc == -1) {
		return rc;
	}

	rc = ipmi_sdr_get(intf, reserve_id, data, 0x00);


	
	return rc;

}

int ipmi_build_sdr_next(struct ipmi_intf *intf, uint8_t *sdr_in, uint8_t *data)
{
	uint16_t reserve_id, record_id;
	uint8_t rc;
	SDRType *sdr_out;
	struct ipmi_rs *rsp;
	struct ipmi_rq req;
	uint8_t msg_data[6];




	memset(&req, 0, sizeof (req));
//	printf("$$$$$$$$$$$$Current ID for get_next func= %02x%02x\n", sdr_in[1], sdr_in[0]);



	/* Get next record ID */
	memset(msg_data, 0, 6);
	msg_data[0] = 0x00;
	msg_data[1] = 0x00;
	msg_data[2] = sdr_in[0]; /* record id */
	msg_data[3] = sdr_in[1]; /* record id */
	msg_data[4] = 0x00;	/* offset */
	msg_data[5] = 0x0f;	/* length */

	req.msg.netfn = IPMI_NETFN_STORAGE;
	req.msg.cmd = GET_SDR;
	req.msg.data = msg_data;
	req.msg.data_len = 6;

	rsp = intf->sendrecv(intf, &req);
	if (rsp == NULL) {
//			lprintf(LOG_ERR, " Error getting SDR info");
			return -1;
		} else if (rsp->ccode > 0) {
//			lprintf(LOG_ERR, " Error getting SDR info: %s",
//				val2str(rsp->ccode, completion_code_vals));
//			printf("sniff1 ccode = %x\n", rsp->ccode);
			return -1;
		}

	record_id = (rsp->data[1] << 8) | rsp->data[0];

	if (record_id != 0xFFFF) {

		rc = ipmi_sdr_reserve(intf, &reserve_id);
		if (rc == -1) {
			return rc;
		}

		rc = ipmi_sdr_get(intf, reserve_id, data, record_id);
		if (rc == -1) {
			return rc;
		}

//		printf("------------------DONE GETTING NEXT RECORD -----------------\n");
	
	
		return 0;
	} else {
//		printf("!!!!!!!!!!!!!last record!\n");
		return 1;
	}


}


int ipmi_sdr_get(struct ipmi_intf *intf, uint16_t reserveId, uint8_t* datapt, uint16_t recordId)
{
	#define MAX_BUF_SIZE 16

	struct ipmi_rs *rsp;
	struct ipmi_rq req;
	uint8_t msg_data[6];
	uint16_t curr_id = 0;

//	uint8_t * datapt;
	uint8_t orgremain;
	uint8_t length;
	uint8_t offset;
	uint8_t remain;
	int i;
	int rc;

//	printf("Entered get_test2\n");
//	datapt = &data[0];


	memset(&req, 0, sizeof (req));


//	printf("SDRget entry recordId = %04x\n", recordId);
	/* Get header info */
	memset(msg_data, 0, 6);
	msg_data[0] = reserveId & 0xff;	
	msg_data[1] = (reserveId >> 8) & 0xff;
	msg_data[2] = recordId & 0xff;
	msg_data[3] = (recordId >> 8) & 0xff;
	msg_data[4] = 0x00;	/* offset */
	msg_data[5] = 0x05;	/* length */

	req.msg.netfn = IPMI_NETFN_STORAGE;
	req.msg.cmd = GET_SDR;
	req.msg.data = msg_data;
	req.msg.data_len = 6;

	

	rsp = intf->sendrecv(intf, &req);
	if (rsp == NULL) {
//			lprintf(LOG_ERR, " Error getting SDR info");
			return -1;
		} else if (rsp->ccode > 0) {
//			lprintf(LOG_ERR, " Error getting SDR info: %s",
//				val2str(rsp->ccode, completion_code_vals));
//			printf("sniff3 ccode = %x\n", rsp->ccode);
			return -1;
		}

	memcpy((uint8_t*)datapt, (uint8_t*)rsp->data + 2, 5);


/*	for (i = 0; i < 16; i++) {
		printf("Byte %d = %x\n", i+1, datapt[i]);
	}
	printf("\n");*/
	orgremain = datapt[4];
	remain = datapt[4];
	offset = 5;
	while (remain > 0) {
		if (remain  < MAX_BUF_SIZE) {
			length = remain;
		} else {
			length = MAX_BUF_SIZE;
		}
		memset(msg_data, 0, 6);
		msg_data[0] = reserveId & 0xff;	
		msg_data[1] = (reserveId >> 8) & 0xff;
		msg_data[2] = recordId & 0xff;
		msg_data[3] = (recordId >> 8) & 0xff;
		msg_data[4] = offset;	/* offset */
		msg_data[5] = length;	/* length */

		req.msg.netfn = IPMI_NETFN_STORAGE;
		req.msg.cmd = GET_SDR;
		req.msg.data = msg_data;
		req.msg.data_len = 6;

		rsp = intf->sendrecv(intf, &req);
		if (rsp == NULL) {
//			lprintf(LOG_ERR, " Error getting SDR info");
			return -1;
		} else if (rsp->ccode > 0) {
//			lprintf(LOG_ERR, " Error getting SDR info: %s",
//				val2str(rsp->ccode, completion_code_vals));
//			printf("sniff2 ccode = %x\n", rsp->ccode);
			return -1;
		}

		memcpy(datapt+offset, rsp->data + 2, length);

		offset += length;
		remain -= length;


	}

/*	if (recordId == 0x56) {
	for (i = 0; i < orgremain; i++) {
		printf("Byte %d = %x\n", i+1, datapt[i]);
	}
	printf("\n");
	}*/



 rc = 0;
	return rc;

}

SDRType* sdr_get_first()
{
	currentSDR = 0;
	return ((SDRType*)(SDR[0].data));
}

SDRType* sdr_get_next(SDRType* pPreviousSDR)
{
	if ((currentSDR + 1) < totalSDRCount) {
		currentSDR++;
		return ((SDRType*)(SDR[currentSDR].data));
	} else {
		return NULL;
	}
}


/*************************************************************************
**************************************************************************/










static int
ipmi_sel_clear(struct ipmi_intf * intf)
{
	struct ipmi_rs * rsp;
	struct ipmi_rq req;
	uint16_t reserve_id;
	uint8_t msg_data[6];

	reserve_id = ipmi_sel_reserve(intf);
	if (reserve_id == 0)
		return -1;

	memset(msg_data, 0, 6);
	msg_data[0] = reserve_id & 0xff;
	msg_data[1] = reserve_id >> 8;
	msg_data[2] = 'C';
	msg_data[3] = 'L';
	msg_data[4] = 'R';
	msg_data[5] = 0xaa;

	memset(&req, 0, sizeof(req));
	req.msg.netfn = IPMI_NETFN_STORAGE;
	req.msg.cmd = IPMI_CMD_CLEAR_SEL;
	req.msg.data = msg_data;
	req.msg.data_len = 6;

	rsp = intf->sendrecv(intf, &req);
	if (rsp == NULL) {
		lprintf(LOG_ERR, "Unable to clear SEL");
		return -1;
	}
	if (rsp->ccode > 0) {
		lprintf(LOG_ERR, "Unable to clear SEL: %s",
			val2str(rsp->ccode, completion_code_vals));
		return -1;
	}

	printf("Clearing SEL.  Please allow a few seconds to erase.\n");
	return 0;
}

static uint16_t
ipmi_sel_reserve(struct ipmi_intf * intf)
{
	struct ipmi_rs * rsp;
	struct ipmi_rq req;

	memset(&req, 0, sizeof(req));
	req.msg.netfn = IPMI_NETFN_STORAGE;
	req.msg.cmd = IPMI_CMD_RESERVE_SEL;

	rsp = intf->sendrecv(intf, &req);
	if (rsp == NULL) {
		lprintf(LOG_WARN, "Unable to reserve SEL");
		return 0;
	}
	if (rsp->ccode > 0) {
		printf("Unable to reserve SEL: %s",
		       val2str(rsp->ccode, completion_code_vals));
		return 0;
	}

	return (rsp->data[0] | (rsp->data[1] << 8));
}

static int ipmi_sensor_print(SDRType* pSdr, int* pSensorIndex, struct ipmi_intf * intf)
{
    CSDDUSERAPI      pfnApiList;
    int              count;
    short            typeSize;
    short            nameSize;
    short            unitSize;
    unsigned char    readingType;
    unsigned char    type;
    char             szUnits[16];
    long             sensorReading;
    unsigned short   sensorState;
    short            readSize;
    short            stateSize;
    unsigned char    thresholdMask;
    SensorThrInfo    thrData;
    SensorThrStrType thrStrData;
    char             szState[SENSOR_STATE_STR_SIZE];
    short            severity;
    char             szName[SENSOR_NAME_STR_SIZE];
    char             szType[SENSOR_TYPE_STR_SIZE];
    char             szReading[SENSOR_READING_STR_SIZE];
    CSDDSensorList   list;
    int              status  = COMMON_STATUS_BAD_PARAMETER;
	IPMISDR*		pSdrPrint;


    /* Clear API structure */
    memset(&pfnApiList, 0, sizeof(CSDDUSERAPI));

    pfnApiList.GetFirstSDR         = sdr_get_first;
    pfnApiList.GetNextSDR          = sdr_get_next;
    pfnApiList.GetSensorReading    = ipmi_sensor_get_sensor_reading;
    pfnApiList.GetSensorThresholds = ipmi_sensor_get_sensor_thresholds;

    CSDDAttach(&pfnApiList);


    if (NULL != pSdr)
    {
        memset((void*)&list, 0, sizeof(CSDDSensorList));
//		printf("Calling CSDDGetSensorsTobeMonitored\n");
        status = CSDDGetSensorsTobeMonitored(&list, pSdr, NULL, (void*)intf);
        /* A failure here means no sensor present/disable for this SDR */
        if (COMMON_STATUS_SUCCESS == status)
        {
//			printf("Test 2\n");
            /* Decode sensor for this SDR.  The list will give a count
               greater than 1 for shared SDRs.  We only need to use
               the sensor number from our sensor table*/
//			printf("sensor count for this SDR is %02x\n", list.sensorCount);
            for(count = 0; count < list.sensorCount; count++)
            { 
//				printf("Test 3\n");
                typeSize        = SENSOR_TYPE_STR_SIZE;
                nameSize        = SENSOR_NAME_STR_SIZE;
                unitSize        = 16;
                stateSize       = SENSOR_STATE_STR_SIZE;
                readSize        = SENSOR_READING_STR_SIZE;
                /* initialize string buffers */
                szState[0]      = 0;
                szType[0]       = 0;
                szReading[0]    = 0;
                szUnits[0]      = 0;

//				printf("Sensor Number = for list%x\n", list.sensorNumber[count]);
//				printf("Sensor Number = for SDR%x\n", pSdr->);
                /* GetSensor Static Information */
                status = CSDDGetSensorStaticInformation(pSdr,0,&readingType,&type,
                             &nameSize, szName, &typeSize, szType, &unitSize,
                             szUnits, list.sensorNumber[count], (void*)intf);
				

//				printf("Exiting CSDDGetSensorStaticInformation\n");

                /* Did we encounter an error? */
                if (COMMON_STATUS_SUCCESS != status)
                {
					printf("Error1\n");
                    break;
                }
				memset(&sensorReading, 0, sizeof(sensorReading));
                status = CSDDGetSensorDynamicInformation(pSdr, &sensorReading,
                            &sensorState, &readSize, szReading, &stateSize,
                            szState, &severity,  list.sensorNumber[count], (void*)intf);

                /* Did we encounter an error? */
                if (COMMON_STATUS_SUCCESS != status)
                {
					printf("Error getting dynamic information: %x\n", status);
                    break;
                }

                /* Output common decoded data */
				
				(*pSensorIndex) ++;
//				printf("print function sensor # = %x\n", list.sensorNumber[0]);
				pSdrPrint = (IPMISDR*)pSdr;
				if (pSdrPrint->type.type1.entityID != 0x22) {
					
					if (IPMI_READING_TYPE_THRESHOLD == readingType)
					{
						printf("Index                          : %d\n", *pSensorIndex);
						printf("Sensor ID Number               : %d\n", pSdrPrint->type.type1.sensorNum);
						printf("Status                         : %s\n", g_StatusTable[severity]);
						printf("Probe Name                     : %s\n", szName);
						printf("Reading                        : %s %s\n", szReading, szUnits);
						memset((void*)&thrData, 0, sizeof(SensorThrInfo));
						memset((void*)&thrStrData, 0, sizeof(SensorThrStrType));
						thresholdMask = 0;        
					
						status = CSDDGetSensorThresholds(pSdr, &thresholdMask, &thrData,
                                &thrStrData, 0, (void*)intf);

						if (COMMON_STATUS_SUCCESS != status)
						{
							printf("Error3\n");
							break;
						}
               /*     printf("Upper NR:    %s %s\n", thrStrData.unrThr,szUnits);
                    printf("Upper C:     %s %s\n", thrStrData.ucThr,szUnits);
                    printf ("Upper NC:    %s %s\n", thrStrData.uncThr,szUnits);
                    printf ("Lower NC:    %s %s\n", thrStrData.lncThr,szUnits);
                    printf ("Lower C:     %s %s\n", thrStrData.lcThr,szUnits);
                    printf ("Lower NR:    %s %s\n", thrStrData.lnrThr,szUnits);*/

						printf ("Minimum Warning Threshold      : %s %s\n", thrStrData.lncThr,szUnits);
						printf ("Maximum Warning Threshold      : %s %s\n", thrStrData.uncThr,szUnits);
						printf ("Minimum Failure Threshold      : %s %s\n", thrStrData.lcThr,szUnits);
						printf ("Maximum Failure Threshold      : %s %s\n", thrStrData.ucThr,szUnits);


						}
						else
						{
							printf("Index            : %d\n", *pSensorIndex);
							printf("Sensor ID Number : %d\n", pSdrPrint->type.type1.sensorNum);
							printf("Status           : %s\n", szState); 
							printf("Probe Name       : %s\n", szName);
					}
					printf("\n");
				}
			}
        }
    }
    CSDDDetach();
    return status;
}



static int ipmi_sensor(struct ipmi_intf* intf, uint8_t sensorNumIn)
{
	int rc = 0;
	int sensorIndex = 0;
	int* pSensorIndex;
	pSensorIndex = &sensorIndex;
	IPMISDR*        pRecord;
	int i;

	rc = ipmi_sdr_build_table(intf);
	if (rc == -1) {
//		printf("sniff1 rc = %x\n", rc);
		return rc;
	}

	i = 0;
	while (i < totalSDRCount) {
		pRecord = (IPMISDR*)SDR[i].data;
		if (sensorNumIn == 0xff && rc != -1) {
			rc = ipmi_sensor_print((SDRType*)SDR[i].data, &sensorIndex, intf);
		}else if (pRecord->type.type1.sensorNum == sensorNumIn){
			rc = ipmi_sensor_print((SDRType*)SDR[i].data, &sensorIndex, intf);
		}
		i++;
	}




	return rc;
}


static int ipmi_sensor_get(struct ipmi_intf* intf, int argc, char **argv)
{
	int rc = 0;
	int sensorIndex = 0;
	int* pSensorIndex;
	pSensorIndex = &sensorIndex;
	IPMISDR*        pRecord;
	int i;
	int current_arg;
	int currIdInt;
	uint8_t currId;

	rc = ipmi_sdr_build_table(intf);
	if (rc == -1) {
		return rc;
	}

	current_arg = 0;
	while (current_arg < argc) {
		make_int(argv[current_arg],&currIdInt);
		currId = currIdInt;
		i = 0;
		while (i < totalSDRCount) {
			pRecord = (IPMISDR*)SDR[i].data;
			if (pRecord->type.type1.sensorNum == currId){
				rc = ipmi_sensor_print((SDRType*)SDR[i].data, &sensorIndex, intf);
			}
			i++;
		}
		current_arg++;
	}

	


	return rc;
}

/* ipmi_sdr_get_sensor_reading  -  retrieve a raw sensor reading
 *
 * @intf:	ipmi interface
 * @sensor:	sensor id
 *
 * returns ipmi response structure
 */
int
ipmi_sensor_get_sensor_reading(unsigned char sensorOwner, unsigned char sensorNumber,
							   IPMISensorReadingType* sensorReadingData,
							   void* x)
{
	struct ipmi_rq req;
	struct ipmi_rs * rsp;
	struct ipmi_intf * intf;
	int rc = 0;
	uint8_t save_addr;



	intf = ((struct ipmi_intf*) x);

	/* save address */
//	save_addr = intf->target_addr;
//	intf->target_addr = sensorOwner;

	memset(&req, 0, sizeof (req));
	req.msg.netfn = IPMI_NETFN_SE;
	req.msg.cmd = GET_SENSOR_READING;
	req.msg.data = &sensorNumber;
	req.msg.data_len = 1;

	rsp = intf->sendrecv(intf, &req);
	if (rsp == NULL) {
//			lprintf(LOG_ERR, " Error getting sensor reading");
			return 1;
		} else if (rsp->ccode > 0) {
//			lprintf(LOG_ERR, " Error getting sensor reading: %s",
//				val2str(rsp->ccode, completion_code_vals));
			return 1;
		}
//	intf->target_addr = save_addr;
/*	printf("    Sensor owner = %x\n", sensorOwner);
	printf("    Sensor # = %x\n", sensorNumber);
	printf("    Data 2 = %x\n", rsp->data[0]);
	printf("    Data 3 = %x\n", rsp->data[1]);
	printf("    Data 4 = %x\n", rsp->data[2]);*/

	memcpy(sensorReadingData, rsp->data, sizeof(IPMISensorReadingType));

	/* if there is an error transmitting ipmi command, return error*/
	if (rsp->ccode != 0) {
		rc = 1;
	}

	/* if sensor messages are disabled, return error*/
	if ((!(rsp->data[1]& 0xC0)) || ((rsp->data[1] & 0x20))) {
		rc =1;
	}



	return rc;
}

/* ipmi_sdr_get_sensor_thresholds  -  return thresholds for sensor
 *
 * @intf:	ipmi interface
 * @sensor:	sensor number
 *
 * returns pointer to ipmi response
 */
int
ipmi_sensor_get_sensor_thresholds(unsigned char sensorOwner, unsigned char sensorNumber,
								  IPMISensorThresholdType* pThrData,
								  void* x)
{
	struct ipmi_rq req;
	struct ipmi_rs * rsp;
	struct ipmi_intf * intf;
	int rc = 0;

	intf = ((struct ipmi_intf*) x);

	memset(&req, 0, sizeof (req));
	req.msg.netfn = IPMI_NETFN_SE;
	req.msg.cmd = GET_SENSOR_THRESHOLDS;
	req.msg.data = &sensorNumber;
	req.msg.data_len = 1;

	rsp = intf->sendrecv(intf, &req);
	if (rsp == NULL) {
			lprintf(LOG_ERR, " Error getting sensor thresholds");
			return 1;
		} else if (rsp->ccode > 0) {
//			lprintf(LOG_ERR, " Error getting sensor thresholds: %s",
//				val2str(rsp->ccode, completion_code_vals));
			return 1;
		}

/*	printf("Completion code = %x\n", rsp->ccode);
	printf("Data 2 = %x\n", rsp->data[0]);
	printf("Data 3 = %x\n", rsp->data[1]);
	printf("Data 4 = %x\n", rsp->data[2]);
	printf("Data 5 = %x\n", rsp->data[3]);
	printf("Data 6 = %x\n", rsp->data[4]);
	printf("Data 7 = %x\n", rsp->data[5]);
	printf("Data 8 = %x\n", rsp->data[6]); */

	memcpy(pThrData, &rsp->data, sizeof(IPMISensorThresholdType));

	if (rsp->ccode != 0) {
		rc = 1;
	}


/*	printf("Threshold Sensor # = %04x\n", sensorNumber);
	printf("Command code = %02x\n", rsp->ccode);
	printf("Sensor reading byte 1 = %02x\n", rsp->data[0]);
	printf("Lower non critical (should be empty) = %02x\n", rsp->data[1]);
	printf("Upper Non Critical Threshold = %02x\n", rsp->data[4]);
	printf("Upper Critical Threshold = %02x\n", rsp->data[5]);*/



	return rc;
	
}

static int ipmi_powermgmt(struct ipmi_intf* intf)
{
	time_t now;
    struct tm* tm;
	struct tm* tm2;
    char* dte;

	struct ipmi_rs * rsp;
	struct ipmi_rq req;
	uint8_t msg_data[2];
	uint32_t cumStartTimeConv;
	uint32_t cumReadingConv;
	uint32_t maxPeakStartTimeConv;
	uint32_t ampPeakTimeConv;
	uint16_t ampReadingConv;
	uint32_t wattPeakTimeConv;
	uint32_t wattReadingConv;


	IPMI_POWER_MONITOR* pwrMonitorInfo;
	char* cumStartTimeStr;
	char* maxPeakStartTimeStr;
	char* ampPeakTimeStr;
	char* wattPeakTimeStr;
	char cumStartTime[25];
	char maxPeakStartTime[25];
	char ampPeakTime[25];
	char wattPeakTime[25];
	float cumReading;
	int ampReading;
	int wattReading;
	int ampReadingRemainder;

	cumStartTimeStr = &cumStartTime[0];
	maxPeakStartTimeStr = &maxPeakStartTime[0];
	ampPeakTimeStr = &ampPeakTime[0];
	wattPeakTimeStr = &wattPeakTime[0];

	now = time(0);
	tm = gmtime(&now);
	dte = asctime(tm);


//	printf("The current time is: %s\n", dte);

	/* get current BMC time info*/
	time_t bmcTime;
	time_t* pBmcTime;
	char* bmcTimeStr;
//	char bmcTimeArray[50];

//	bmcTimeStr = &bmcTimeArray[0];


	pBmcTime = &bmcTime;

	memset(&req, 0, sizeof(req));
	req.msg.netfn = IPMI_NETFN_STORAGE;
	req.msg.cmd = IPMI_CMD_GET_SEL_TIME;

	rsp = intf->sendrecv(intf, &req);
	if (rsp == NULL) {
			lprintf(LOG_ERR, " Error getting BMC time info.\n");
			return -1;
	} 
	if (rsp->ccode != 0) {
		printf("Error getting power management information, return code %x\n", rsp->ccode);
		return -1;
	}

	memcpy(pBmcTime, rsp->data, sizeof(time_t));


	tm2 = gmtime(pBmcTime);
//	strcpy(bmcTimeStr, asctime(tm));

//	printf("The current BMC time is: %s\n", bmcTimeStr);

	/* get powermanagement info*/
	req.msg.netfn = 0x30;
	req.msg.lun = 0x0;
	req.msg.cmd = 0x9c;
	req.msg.data = msg_data;
	req.msg.data_len = 2;


		
	memset(msg_data, 0, 2);
//	printf("##########netfn = %x\n", req.msg.netfn);
	msg_data[0] = 0x07;	
	msg_data[1] = 0x01;




	rsp = intf->sendrecv(intf, &req);
	if (rsp == NULL) {
				lprintf(LOG_ERR, " Error getting power management information.\n");
				return -1;
	} 
	if (rsp->ccode == 0xc1) {
		lprintf(LOG_ERR, " Error getting power management information, command not supported on this system.\n");
		return -1;
	}else if (rsp->ccode != 0) {
		printf("Error getting power management information, return code %x\n", rsp->ccode);
		return -1;
	}


/*	int i;
	for(i=0;i<24;i++)
	{
		printf("byte %d: %x\n",i+2, rsp->data[i]);
	}*/

	pwrMonitorInfo = (IPMI_POWER_MONITOR*)rsp->data;


/*	cumStartTimeConv = (rsp->data[0]<<24) | (rsp->data[1]<<16) | (rsp->data[2]<<8) | rsp->data[3];
	cumReadingConv = (rsp->data[4]<<24) | (rsp->data[5]<<16) | (rsp->data[6]<<8) | rsp->data[7];
//	printf("raw reading = %x\n", pwrMonitorInfo->cumReading);
//	printf("correct reading = %x\n", cumReadingConv);
	maxPeakStartTimeConv = (rsp->data[8]<<24) | (rsp->data[9]<<16) | (rsp->data[10]<<8) | rsp->data[11];
	ampPeakTimeConv = (rsp->data[12]<<24) | (rsp->data[13]<<16) | (rsp->data[14]<<8) | rsp->data[15];
	ampReadingConv = (rsp->data[16]<<8) | rsp->data[17];
	wattPeakTimeConv = (rsp->data[18]<<24) | (rsp->data[19]<<16) | (rsp->data[20]<<8) | rsp->data[21];
	wattReadingConv = (rsp->data[22]<<8) | rsp->data[23];*/

	cumStartTimeConv = pwrMonitorInfo->cumStartTime;
	cumReadingConv = pwrMonitorInfo->cumReading;
	maxPeakStartTimeConv = pwrMonitorInfo->maxPeakStartTime;
	ampPeakTimeConv = pwrMonitorInfo->ampPeakTime;
	ampReadingConv = pwrMonitorInfo->ampReading;
	wattPeakTimeConv = pwrMonitorInfo->wattPeakTime;
	wattReadingConv = pwrMonitorInfo->wattReading;


/*	printf("cumStartTime = %x\n", cumStartTimeConv);
	printf("cumReading = %x\n", cumReadingConv);
	printf("maxPeakStartTime = %x\n", maxPeakStartTimeConv);
	printf("ampPeakTime = %x\n", ampPeakTimeConv);
	printf("ampReading = %x\n", ampReadingConv);
	printf("wattPeakTime = %x\n", wattPeakTimeConv);
	printf("wattReading = %x\n", wattReadingConv);*/
	ipmi_time_to_str(cumStartTimeConv, cumStartTimeStr);
	ipmi_time_to_str(maxPeakStartTimeConv, maxPeakStartTimeStr);
	ipmi_time_to_str(ampPeakTimeConv, ampPeakTimeStr);
	ipmi_time_to_str(wattPeakTimeConv, wattPeakTimeStr);

	now = time(0);

	int round;
	int round2;
	int remainder;


/*	ipmi_time_to_str(now, cumStartTimeStr);
	ipmi_time_to_str(now, maxPeakStartTimeStr);
	ipmi_time_to_str(now, ampPeakTimeStr);
	ipmi_time_to_str(now, wattPeakTimeStr);*/

//	cumReading = cumReadingConv;
//	cumReadingConv = /*cumReadingConv/100*/15872;
	remainder = (cumReadingConv % 1000);
	cumReadingConv = cumReadingConv / 1000;
//	printf("cumReading = %d\n", cumReadingConv);
	remainder = (remainder + 50) / 100;
//	remainder = (cumReadingConv % 1000);
//	printf("remainder = %d\n", remainder);

//	printf("478 % 10 = %d\n", (478%10));
	

	ampReading = ampReadingConv;
	ampReading = ampReading/10;
	ampReadingRemainder = ampReading%10;

	wattReading = wattReadingConv;

	printf("Power Tracking Statistics\n");
	printf("Statistic                  : Energy Consumption\n");
	printf("Measurement Start Time     : %s", cumStartTimeStr);
	printf("Measurement Finish Time    : %s", /*bmcTimeStr*/asctime(tm));
	printf("Reading                    : %d.%d kWh\n\n", cumReadingConv, remainder);

	printf("Statistic                  : System Peak Power\n");
	printf("Measurement Start Time     : %s", maxPeakStartTimeStr);
 	printf("Peak Time                  : %s", wattPeakTimeStr);
	printf("Peak Reading               : %d W\n\n", wattReading);

    printf("Statistic                  : System Peak Amperage\n");
	printf("Measurement Start Time     : %s", maxPeakStartTimeStr);
 	printf("Peak Time                  : %s", ampPeakTimeStr);
	printf("Reading                    : %d.%d A\n", ampReading, ampReadingRemainder);

/*	int d = 0xb;
	float f = 0xBC;
	float g = f/1000;

	printf("\n\nPrintf hex = %x, decimal = %d, float = %f\n", d, d, (float)0xBC);*/


	return 0;

}

static int
ipmi_powermgmt_clear(struct ipmi_intf* intf, int8_t clearValue)
{
	struct ipmi_rs * rsp;
	struct ipmi_rq req;
	uint8_t	clearType;
	uint8_t msg_data[3];

	if (clearValue) {
		clearType = 2;
	} else {
		clearType = 1;
	}

	/* clear powermanagement info*/
	req.msg.netfn = 0x30;
	req.msg.lun = 0;
	req.msg.cmd = 0x9d;
	req.msg.data = msg_data;
	req.msg.data_len = 3;

		
	memset(msg_data, 0, 3);
	msg_data[0] = 0x07;
	msg_data[1] = 0x01;
	msg_data[2] = clearType;



	rsp = intf->sendrecv(intf, &req);
	if (rsp == NULL) {
				lprintf(LOG_ERR, " Error clearing power values.\n");
				return -1;
	} else if (rsp->ccode == 0xc1) {
		lprintf(LOG_ERR, " Error clearing power values, command not supported on this system.\n");
		return -1;
	} else if (rsp->ccode != 0){
		lprintf(LOG_ERR, " Error clearing power values: %s",
				val2str(rsp->ccode, completion_code_vals));
		return -1;
	}
//	printf("ccode = %x\n", rsp->ccode);

/*	if (rsp->ccode != 0) {
		printf("Error clearing power values\n");
	}*/
}

static void
ipmi_time_to_str(time_t rawTime, char* strTime)
{
	struct tm* tm;
	char *temp;

	tm = gmtime(&rawTime);
	temp = asctime(tm);

	strcpy(strTime,temp);

	
}



static void
ipmi_sel_usage(void)
{
	lprintf(LOG_NOTICE, "");
	lprintf(LOG_NOTICE, "   sel info");
	lprintf(LOG_NOTICE, "      Show information about System Event Log (SEL) and its contents");
	lprintf(LOG_NOTICE, "");
	lprintf(LOG_NOTICE, "   sel clear");
	lprintf(LOG_NOTICE, "      Clear the contents of the SEL");
	lprintf(LOG_NOTICE, "");
	lprintf(LOG_NOTICE, "   sel time");
	lprintf(LOG_NOTICE, "      get");
	lprintf(LOG_NOTICE, "	      Displays the SEL clock's current time");
	lprintf(LOG_NOTICE, "      set");
	lprintf(LOG_NOTICE, "         Set the SEL clock in the form 'MM/DD/YYYY HH:MM:SS' ");
	lprintf(LOG_NOTICE, "");
	lprintf(LOG_NOTICE, "   sel list");
	lprintf(LOG_NOTICE, "      When used without any arguments, entire contents of SEL are displayed");
	lprintf(LOG_NOTICE, "      first <count>");
	lprintf(LOG_NOTICE, "         Displays first (least recent) count entries in SEL. Count of 0 will display all entries");
	lprintf(LOG_NOTICE, "      last <count>");
	lprintf(LOG_NOTICE, "         Displays last (most recent) count entries in SEL. Count of 0 will display all entries");
	lprintf(LOG_NOTICE, "");
}

static void
ipmi_sensor_usage(void)
{
	lprintf(LOG_NOTICE, "");
	lprintf(LOG_NOTICE, "   sensor list");
	lprintf(LOG_NOTICE, "      Lists status, reading, and thresholds for sensors");
	lprintf(LOG_NOTICE, "");
	lprintf(LOG_NOTICE, "   sensor get <id>");
	lprintf(LOG_NOTICE, "      Shows information for specified sensor");
	lprintf(LOG_NOTICE, "");
	lprintf(LOG_NOTICE, "   sensor thresh <id> <threshold> <setting>");
	lprintf(LOG_NOTICE, "      Allows the value of threshold on specified sensor id to be changed");
	lprintf(LOG_NOTICE, "      Valid thresholds:");
	lprintf(LOG_NOTICE, "		unr	Upper Non-Recoverable");
	lprintf(LOG_NOTICE, "		ucr	Upper Critical");
	lprintf(LOG_NOTICE, "		unc	Upper Non-Critical");
	lprintf(LOG_NOTICE, "		lnc	Lower Non-Critical");
	lprintf(LOG_NOTICE, "		lcr	Lower Non-Critical");
	lprintf(LOG_NOTICE, "		lnr	Lower Non-Recoverable");
	lprintf(LOG_NOTICE, "");
}

static void
ipmi_powermonitor_usage(void)
{
	lprintf(LOG_NOTICE, "");
	lprintf(LOG_NOTICE, "   powermonitor");
	lprintf(LOG_NOTICE, "      Shows power tracking statistics ");
	lprintf(LOG_NOTICE, "");
	lprintf(LOG_NOTICE, "   powermonitor clear cumulativepower");
	lprintf(LOG_NOTICE, "      Reset cumulative power reading");
	lprintf(LOG_NOTICE, "");
	lprintf(LOG_NOTICE, "   powermonitor clear peakpower");
	lprintf(LOG_NOTICE, "      Reset peak power reading");
	lprintf(LOG_NOTICE, "");
}
	
static void
ipmi_sysinfo_usage(void)
{
	lprintf(LOG_NOTICE, "");
	lprintf(LOG_NOTICE, "   sysinfo");
	lprintf(LOG_NOTICE, "      Show system information");
}

static void
ipmi_lcd_usage(void)
{
	lprintf(LOG_NOTICE, "");
	lprintf(LOG_NOTICE, "   lcd set {none}|{default}|{custom <text>}");
	lprintf(LOG_NOTICE, "      Set LCD text displayed during non-fault conditions");
	lprintf(LOG_NOTICE, "");
	lprintf(LOG_NOTICE, "   lcd info");
	lprintf(LOG_NOTICE, "      Show LCD text that is displayed during non-fault conditions");
	lprintf(LOG_NOTICE, "");
}

static void
usage(void)
{
	lprintf(LOG_NOTICE, "");
	lprintf(LOG_NOTICE, "usage: delloem <command> [option...]");
	lprintf(LOG_NOTICE, "");
	lprintf(LOG_NOTICE, "commands:");
	lprintf(LOG_NOTICE, "    sysinfo");
	lprintf(LOG_NOTICE, "    lcd");
	lprintf(LOG_NOTICE, "    sel");
	lprintf(LOG_NOTICE, "    sensor");
	lprintf(LOG_NOTICE, "    powermonitor");
	lprintf(LOG_NOTICE, "");
	lprintf(LOG_NOTICE, "For help on individual commands type:");
	lprintf(LOG_NOTICE, "delloem <command> help");

}

int
ipmi_delloem_main(struct ipmi_intf * intf, int argc, char ** argv)
{
	int rc = 0;
	int current_arg = 0;

	if (argc == 0 || strncmp(argv[0], "help\0", 5) == 0) 
	{
		usage();
		return 0;
	}

	// ----------------------------------------
	// ipmitool delloem lcd command begins here
	if (strncmp(argv[current_arg], "lcd\0", 4) == 0) {
		current_arg++;
		if (argc < current_arg) {usage();return -1;}

		// ipmitool delloem lcd info
		if (argc == 1) {
			rc = ipmi_lcd_get_info(intf);
		}
		else if (strncmp(argv[current_arg], "info\0", 5) == 0) {
//			printf("info parameter\n");
			rc = ipmi_lcd_get_info(intf);
		}

		// ipmitool delloem lcd set
		else if (strncmp(argv[current_arg], "set\0", 4) == 0) {
			uint8_t line_number = 0;

			current_arg++;
			if (argc <= current_arg) {ipmi_lcd_usage();return -1;}

			if (strncmp(argv[current_arg], "line\0", 5) == 0) {
				current_arg++;
				if (argc <= current_arg) {usage();return -1;}
				line_number = (uint8_t)strtoul(argv[current_arg], NULL, 0);
				current_arg++;
				if (argc <= current_arg) {usage();return -1;}
			}

			if (strncmp(argv[current_arg], "none\0", 5) == 0) {
				rc = ipmi_lcd_configure (intf, IPMI_DELL_LCD_CONFIG_NONE, 0, NULL);
			}
			else if (strncmp(argv[current_arg], "default\0", 8) == 0) {
				rc = ipmi_lcd_configure (intf, IPMI_DELL_LCD_CONFIG_DEFAULT, 0, NULL);

			} else if (strncmp(argv[current_arg], "custom\0", 7) == 0) {
				current_arg++;
				if (argc <= current_arg) {ipmi_lcd_usage();return -1;}
				rc = ipmi_lcd_configure (intf, IPMI_DELL_LCD_CONFIG_USER_DEFINED, line_number, argv[current_arg]);
			} else if (strncmp(argv[current_arg], "help\0", 5) == 0) {
				ipmi_lcd_usage();
			}
			else {
				ipmi_lcd_usage();
				return -1;
			}
		} else {
			ipmi_lcd_usage();
			return -1;
		}
	// start sysinfo
	} else if (strncmp(argv[current_arg], "sysinfo\0", 8) == 0) {
		current_arg++;

//		printf ("argc %d    current_arg %d\n", argc, current_arg);
		if (argc <= current_arg) {
			rc = ipmi_sysinfo_id(intf, 0);
//			rc = ipmi_sysinfo_fru(intf, 0);

		} else {
			ipmi_sysinfo_usage();
		}
	// start sel processing
	} else if (strncmp(argv[current_arg], "sel\0", 4) == 0) {
		current_arg++;
//		printf("argc = %d\n", argc);
		if (argc == 1) {
//			printf("Entered sel w/ no arguments\n");
			rc = ipmi_sel_get_info(intf);
		} else if (strncmp(argv[current_arg], "info\0", 5) == 0) {
//			printf("Entering the 'sel' portion\n");
//			rc = ipmi_sel_print(intf, 0xFFFF);
			rc = ipmi_sel_get_info(intf);
		} else if (strncmp(argv[current_arg], "clear\0", 6) == 0) {
			rc = ipmi_sel_clear(intf);
		} else if (strncmp(argv[current_arg], "list\0", 5) == 0) {
			current_arg++;
			int count = 0;
//			printf("argc = %d\n", argc);
//			printf("argv[2] = %s\n", argv[2]);
			char *countstr = NULL;
			if (argv[current_arg] == NULL) {
				rc = ipmi_sel_print(intf, 0xFFFF,0);
			} 
			else if ((strncmp(argv[current_arg], "first\0", 6) == 0) ||
					 (argc == 3)) {
				current_arg++;
				countstr = argv[current_arg];
				if(argc == 3){
					countstr = argv[2];
				}
				if (countstr) {
					if (make_int(countstr,&count) < 0) {
						lprintf(LOG_ERR, "Numeric argument required; got '%s'",
								countstr);
						return -1;
					}
				}

				if (count == 0) {
					rc = ipmi_sel_print(intf, 0xFFFF,0);
				} else {
					rc = ipmi_sel_print(intf, count, 1);
				}
			} else if (strncmp(argv[current_arg], "last\0", 5) == 0) {
				current_arg++;
				countstr = argv[current_arg];
				if (countstr) {
					if (make_int(countstr,&count) < 0) {
						lprintf(LOG_ERR, "Numeric argument required; got '%s'",
								countstr);
						return -1;
					}
				}
				if (count == 0) {
					rc = ipmi_sel_print(intf, 0xFFFF,0);
				} else {
					rc = ipmi_sel_print(intf, count, 2);
				}

			}
			
		} else if (strncmp(argv[current_arg], "time\0", 5) == 0) {
			current_arg++;
			if (argc == 2) {
				rc = ipmi_sel_get_time(intf);
			}
			else if (strncmp(argv[current_arg], "get\0", 4) == 0) {
//				printf("Entered get time\n");
				rc = ipmi_sel_get_time(intf);
			} else if (strncmp(argv[current_arg], "set\0", 4) == 0) {
				if (argc < 4) {
					ipmi_sel_usage();
				} else {
		
					rc = ipmi_sel_set_time(intf, argv[3]);
				}
			} /*else {
				usage();
				return -1;
			}*/
			
		} else {
			ipmi_sel_usage();
			return -1;
		}
	// Sensor report processing
	} else if (strncmp(argv[current_arg], "sensor\0", 7) == 0) {
		current_arg++;
		if (argc == 1) {
			rc = ipmi_sensor(intf, 0xff);
		}
		else if (strncmp(argv[current_arg], "list\0", 5) == 0) {
			rc = ipmi_sensor(intf, 0xff);
		} else if (strncmp(argv[current_arg], "get\0", 4) == 0) {
			current_arg++;
			if (argv[current_arg] == NULL) {
				ipmi_sensor_usage();
				return -1;
			}
			int currIdInt;
			uint8_t currId;
			make_int(argv[current_arg],&currIdInt);
			currId = currIdInt;
//			rc = ipmi_sensor(intf, currId);
			rc = ipmi_sensor_get(intf, argc - 2, &argv[2]);
		} else if (strncmp(argv[current_arg], "thresh\0", 7) == 0) {
			current_arg++;
			rc = ipmi_sensor_set_threshold(intf, argc - 2, &argv[current_arg]);
		} else {
			ipmi_sensor_usage();
		}
	// Powermanagement report processing
	} else if (strncmp(argv[current_arg], "powermonitor\0", 13) == 0) {
		current_arg++;
		if (argc == 1) {
			rc = ipmi_powermgmt(intf);
		} else if (strncmp(argv[current_arg], "status\0", 7) == 0) {
			rc = ipmi_powermgmt(intf);
		} else if (strncmp(argv[current_arg], "clear\0", 6) == 0) {
			current_arg++;
			if (argv[current_arg] == NULL) {
				ipmi_powermonitor_usage();
				return -1;
			}else if (strncmp(argv[current_arg], "peakpower\0", 10) == 0) {
				rc = ipmi_powermgmt_clear(intf, 1);
			} else if (strncmp(argv[current_arg], "cumulativepower\0", 16) == 0) {
				rc = ipmi_powermgmt_clear(intf, 0);
			} else {
				ipmi_powermonitor_usage();
				return -1;
			}
			
		} else {
			ipmi_powermonitor_usage();
			return -1;
		}
		
	} else if (strncmp(argv[current_arg], "test\0", 5) == 0) {
		rc = ipmi_sdr_build_table(intf);
	}
	else {
		usage();
		return -1;
	}

	return rc;
}

