/****************************************************************************
  Copyright (c) 2006, Dell Inc
  All rights reserved.
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:
  - Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer.
    
  - Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution. 
  - Neither the name of Dell Inc nor the names of its contributors
    may be used to endorse or promote products derived from this software 
    without specific prior written permission. 
  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE. 
 
 
*****************************************************************************/

#include <ipmitool/cssipmi.h>
#include <ipmitool/cssapi.h>
#include <ipmitool/cssstat.h>
#include <ipmitool/csssupt.h>

static char g_UnknownState[] = "unknown";
static char g_NormalState[]  = "normal";
static char g_WarningState[] = "warning";
static char g_FailedState[]  = "failed";
static char g_NRState[]      = "non-recoverable";


/* Sensor Units String Table */
char* g_SensorUnitsTable[MAX_UNITS_TYPE_NAMES] =
{
    " ","C","F","K","V","AMP","Watts","Joules","Coulombs","VA","Nits",
    "lumen","lux","Candela","kPa","PSI","N","CFM","RPM","Hz","us","ms","sec",
    "minute","hour","day","week","mil","in","ft","cu in","cu ft","mm","cm",
    "m","cu cm","cu m","liters","fluid oz","radians","steradians","revolutions",
    "cycles","gravities","oz","lb","ft-lb","oz-in","gauss","gilberts","henry",
    "millihenry","farad","ufarad","ohms","siemens","mole","becquerel","PPM",
    "reserved","Decibels","DbA","DbC","gray","sievert","color K","bit",
    "kilobit","megabit","gigabit","byte","kilobyte","megabyte","gigabyte",
    "word","dword","qword","line","hit","miss","retry","reset","overrun",
    "underrun","collision","packets","messages","characters","error", 
    "correctable error","uncorrectable error","fatal error","grams"
};


/****************************************************************************
  Generic reading/event tables
****************************************************************************/


/* Threshold type */
#define SENSOR_GENERICTYPE1_SIZE (13)
SensorStateText g_SensorGenericType1[SENSOR_GENERICTYPE1_SIZE] =
{
    {CSS_SEVERITY_NORMAL,           g_NormalState},
    {CSS_SEVERITY_WARNING,          g_WarningState},
    {CSS_SEVERITY_NORMAL,           g_NormalState},
    {CSS_SEVERITY_CRITICAL,         g_FailedState},
    {CSS_SEVERITY_WARNING,          g_WarningState},
    {CSS_SEVERITY_NONRECOVERABLE,   g_NRState},
    {CSS_SEVERITY_CRITICAL,         g_FailedState},
    {CSS_SEVERITY_NORMAL,           g_NormalState},
    {CSS_SEVERITY_WARNING,          g_WarningState},
    {CSS_SEVERITY_WARNING,          g_WarningState},
    {CSS_SEVERITY_CRITICAL,         g_FailedState},
    {CSS_SEVERITY_CRITICAL,         g_FailedState},
    {CSS_SEVERITY_NONRECOVERABLE,   g_NRState}
};

/* Discrete type 2  */
#define SENSOR_GENERICTYPE2_SIZE (4) 
SensorStateText g_SensorGenericType2[SENSOR_GENERICTYPE2_SIZE] =
{
    {CSS_SEVERITY_UNKNOWN,   g_UnknownState}, 
    {CSS_SEVERITY_NORMAL,    "idle"}, 
    {CSS_SEVERITY_NORMAL,    "active"},           
    {CSS_SEVERITY_WARNING,   "busy"}
};

/* Discrete type 3  */
#define SENSOR_GENERICTYPE3_SIZE (3)
SensorStateText g_SensorGenericType3[SENSOR_GENERICTYPE3_SIZE] =
{
    {CSS_SEVERITY_NORMAL,    "good"},
    {CSS_SEVERITY_NORMAL,    "good"},     
    {CSS_SEVERITY_CRITICAL,  "bad"}
};

/* Discrete type 4  */ 
#define SENSOR_GENERICTYPE4_SIZE (3)
SensorStateText g_SensorGenericType4[SENSOR_GENERICTYPE4_SIZE] =
{ 
    {CSS_SEVERITY_NORMAL,    "good"},   /* severity matches OMSA */
    {CSS_SEVERITY_NORMAL,    "good"},
    {CSS_SEVERITY_WARNING,   "bad"}
};

/* Discrete type 5  */  
#define SENSOR_GENERICTYPE5_SIZE (3)
SensorStateText g_SensorGenericType5[SENSOR_GENERICTYPE5_SIZE] =
{
    {CSS_SEVERITY_UNKNOWN,   g_UnknownState},    /* severity matches OMSA */
    {CSS_SEVERITY_NORMAL,    "good"},
    {CSS_SEVERITY_WARNING,   "exceeded"}
};

/* Discrete type 6 */
#define SENSOR_GENERICTYPE6_SIZE (3)
SensorStateText g_SensorGenericType6[SENSOR_GENERICTYPE6_SIZE] =      
{ 
    {CSS_SEVERITY_UNKNOWN,   g_UnknownState}, /* severity matches OMSA */
    {CSS_SEVERITY_NORMAL,    "met"},
    {CSS_SEVERITY_WARNING,   "lags"}
};
  
/* Discrete type 7 */
#define SENSOR_GENERICTYPE7_SIZE (10)  
SensorStateText g_SensorGenericType7[SENSOR_GENERICTYPE7_SIZE] =
{
    {CSS_SEVERITY_UNKNOWN,          g_UnknownState}, 
    {CSS_SEVERITY_NORMAL,           g_NormalState},
    {CSS_SEVERITY_WARNING,          g_WarningState},
    {CSS_SEVERITY_CRITICAL,         g_FailedState},
    {CSS_SEVERITY_NONRECOVERABLE,   g_NRState},
    {CSS_SEVERITY_WARNING,          g_WarningState},
    {CSS_SEVERITY_CRITICAL,         g_FailedState},
    {CSS_SEVERITY_NONRECOVERABLE,   g_NRState},
    {CSS_SEVERITY_NORMAL,           "monitor"},
    {CSS_SEVERITY_NORMAL,           "informational"}
};

/* Discrete type 8  */ 
#define SENSOR_GENERICTYPE8_SIZE (3)
SensorStateText g_SensorGenericType8[SENSOR_GENERICTYPE8_SIZE] =     
{    
    {CSS_SEVERITY_UNKNOWN,   g_UnknownState},     /* severity matches OMSA */
    {CSS_SEVERITY_CRITICAL,  "removed"},
    {CSS_SEVERITY_NORMAL,    "inserted"}
};

/* Discrete type 9  */ 
#define SENSOR_GENERICTYPE9_SIZE (3) 
SensorStateText g_SensorGenericType9[SENSOR_GENERICTYPE9_SIZE] =
{ 
    {CSS_SEVERITY_UNKNOWN,   g_UnknownState},   /* severity matches OMSA */
    {CSS_SEVERITY_CRITICAL,  "disabled"},
    {CSS_SEVERITY_NORMAL,    "enabled"}
};
    
/* Discrete type A  */ 
#define SENSOR_GENERICTYPEA_SIZE (10) 
SensorStateText g_SensorGenericTypeA[SENSOR_GENERICTYPEA_SIZE] =
{ 
    {CSS_SEVERITY_UNKNOWN,   g_UnknownState},  
    {CSS_SEVERITY_NORMAL,    "running"},
    {CSS_SEVERITY_NORMAL,    "in test"},
    {CSS_SEVERITY_CRITICAL,  "power off"},
    {CSS_SEVERITY_NORMAL,    "on line"},
    {CSS_SEVERITY_CRITICAL,  "off line"},
    {CSS_SEVERITY_NORMAL,    "off duty"},
    {CSS_SEVERITY_WARNING,   "degraded"},
    {CSS_SEVERITY_NORMAL,    "power save"},
    {CSS_SEVERITY_CRITICAL,  "install error"}
};

/* Discrete type B Redundancy  */ 
#define SENSOR_GENERICTYPEB_SIZE (4) 
SensorStateText g_SensorGenericTypeB[SENSOR_GENERICTYPEB_SIZE] =
{ 
    {CSS_SEVERITY_NORMAL,    "full"},
    {CSS_SEVERITY_NORMAL,    "full"},
    {CSS_SEVERITY_CRITICAL,  "lost"},
    {CSS_SEVERITY_WARNING,   "degraded"}
};
    
/* Discrete type C  */ 
#define SENSOR_GENERICTYPEC_SIZE (5)
SensorStateText g_SensorGenericTypeC[SENSOR_GENERICTYPEC_SIZE] =
{
    {CSS_SEVERITY_UNKNOWN,   g_UnknownState},
    {CSS_SEVERITY_NORMAL,    "D0"},
    {CSS_SEVERITY_NORMAL,    "D1"},
    {CSS_SEVERITY_NORMAL,    "D2"},
    {CSS_SEVERITY_NORMAL,    "D3"}
};


/* Generic  access Table */
SensorStateElement g_SensorGenericTable[] =
{
    {IPMI_READING_TYPE_THRESHOLD,       SENSOR_GENERICTYPE1_SIZE, g_SensorGenericType1},
    {IPMI_READING_TYPE_DISCRETE,        SENSOR_GENERICTYPE2_SIZE, g_SensorGenericType2},
    {IPMI_READING_TYPE_DIG_DISCRETE3,   SENSOR_GENERICTYPE3_SIZE, g_SensorGenericType3},
    {IPMI_READING_TYPE_DIG_DISCRETE4,   SENSOR_GENERICTYPE4_SIZE, g_SensorGenericType4},
    {IPMI_READING_TYPE_DIG_DISCRETE5,   SENSOR_GENERICTYPE5_SIZE, g_SensorGenericType5},
    {IPMI_READING_TYPE_DIG_DISCRETE6,   SENSOR_GENERICTYPE6_SIZE, g_SensorGenericType6},
    {IPMI_READING_TYPE_DIG_DISCRETE7,   SENSOR_GENERICTYPE7_SIZE, g_SensorGenericType7},
    {IPMI_READING_TYPE_DIG_DISCRETE8,   SENSOR_GENERICTYPE8_SIZE, g_SensorGenericType8},
    {IPMI_READING_TYPE_DIG_DISCRETE9,   SENSOR_GENERICTYPE9_SIZE, g_SensorGenericType9},
    {IPMI_READING_TYPE_DISCRETE0A,      SENSOR_GENERICTYPEA_SIZE, g_SensorGenericTypeA},
    {IPMI_READING_TYPE_REDUNDANCY,      SENSOR_GENERICTYPEB_SIZE, g_SensorGenericTypeB},
    {IPMI_READING_TYPE_DISCRETE0C,      SENSOR_GENERICTYPEC_SIZE, g_SensorGenericTypeC}
};

/****************************************************************************
  Specific reading/event tables
****************************************************************************/
 
/* 5 intrusion */ 
#define SENSOR_SPECIFICTYPE5_SIZE (8)
SensorStateText g_SensorSpecificType5[SENSOR_SPECIFICTYPE5_SIZE] =
{
    {CSS_SEVERITY_NORMAL,    "no intrusion"},
    {CSS_SEVERITY_CRITICAL,  "chassis intrusion"},
    {CSS_SEVERITY_CRITICAL,  "drive bay intrusion"},
    {CSS_SEVERITY_CRITICAL,  "I/O card area intrusion"},
    {CSS_SEVERITY_CRITICAL,  "processor area intrusion"},
    {CSS_SEVERITY_CRITICAL,  "LAN disconnect"},
    {CSS_SEVERITY_CRITICAL,  "unauthorized dock"},
    {CSS_SEVERITY_CRITICAL,  "FAN area intrusion"}
};

/* 6 Security Violation */   
#define SENSOR_SPECIFICTYPE6_SIZE (7) 
SensorStateText g_SensorSpecificType6[SENSOR_SPECIFICTYPE6_SIZE] =
{
    {CSS_SEVERITY_NORMAL,   "no violations"},
    {CSS_SEVERITY_WARNING,  "violation"},
    {CSS_SEVERITY_WARNING,  "pre-boot user password violation"},
    {CSS_SEVERITY_WARNING,  "pre-boot setup password violation"},
    {CSS_SEVERITY_WARNING,  "pre-boot network boot password violation"},
    {CSS_SEVERITY_WARNING,  "password violation"},
    {CSS_SEVERITY_WARNING,  "out-of-band password violation"}
};

/* 7 Processor */  
#define SENSOR_SPECIFICTYPE7_SIZE (12)  
SensorStateText g_SensorSpecificType7[SENSOR_SPECIFICTYPE7_SIZE] =
{
    {CSS_SEVERITY_UNKNOWN,   "absent"},
    {CSS_SEVERITY_CRITICAL,  "IERR"},
    {CSS_SEVERITY_CRITICAL,  "thermal trip"},
    {CSS_SEVERITY_CRITICAL,  "FRB1/BIST failure"},
    {CSS_SEVERITY_CRITICAL,  "FRB2/Hang in POST failure"},
    {CSS_SEVERITY_CRITICAL,  "FRB3 startup initialization"},
    {CSS_SEVERITY_CRITICAL,  "configuration"},
    {CSS_SEVERITY_CRITICAL,  "uncorrectable CPU complex"},
    {CSS_SEVERITY_NORMAL,    "present and OK"},
    {CSS_SEVERITY_WARNING,   "disabled"},
    {CSS_SEVERITY_NORMAL,    "terminator present"},
    {CSS_SEVERITY_WARNING,   "throttled"}
}; 
    
/* 8 Power Supply */
#define SENSOR_SPECIFICTYPE8_SIZE (8)    
SensorStateText g_SensorSpecificType8[SENSOR_SPECIFICTYPE8_SIZE] =
{
    {CSS_SEVERITY_CRITICAL,  "absent"},             /* matches OMSA */
    {CSS_SEVERITY_NORMAL,    "present and OK"},
    {CSS_SEVERITY_CRITICAL,  "failed"},
    {CSS_SEVERITY_WARNING,   "predictive failure"},
    {CSS_SEVERITY_CRITICAL,  "input lost"},
    {CSS_SEVERITY_CRITICAL,  "input lost or out of range"},
    {CSS_SEVERITY_CRITICAL,  "input present but out of range"},
    {CSS_SEVERITY_CRITICAL,  "configuration error"}
}; 
    
/* 9 Power Unit */ 
#define SENSOR_SPECIFICTYPE9_SIZE (9)   
SensorStateText g_SensorSpecificType9[SENSOR_SPECIFICTYPE9_SIZE] =
{
    {CSS_SEVERITY_NORMAL,    "on"},   
    {CSS_SEVERITY_NORMAL,    "off"},
    {CSS_SEVERITY_NORMAL,    "cycled"},
    {CSS_SEVERITY_CRITICAL,  "240VA power down error"},
    {CSS_SEVERITY_CRITICAL,  "interlock power down error"},
    {CSS_SEVERITY_CRITICAL,  "power lost"},
    {CSS_SEVERITY_CRITICAL,  "soft power control failed"},
    {CSS_SEVERITY_CRITICAL,  "power unit failed"},
    {CSS_SEVERITY_WARNING,   "predictive failed"}
}; 
    
/* C Memory */
#define SENSOR_SPECIFICTYPEC_SIZE (12)    
SensorStateText g_SensorSpecificTypeC[SENSOR_SPECIFICTYPEC_SIZE] =
{
    {CSS_SEVERITY_UNKNOWN,   "absent"},
    {CSS_SEVERITY_WARNING,   "correctable ECC"},
    {CSS_SEVERITY_CRITICAL,  "uncorrectable ECC"},
    {CSS_SEVERITY_CRITICAL,  "parity"},
    {CSS_SEVERITY_CRITICAL,  "scrub failed"},
    {CSS_SEVERITY_WARNING,   "disabled"},
    {CSS_SEVERITY_CRITICAL,  "ECC logging limit reached"},
    {CSS_SEVERITY_NORMAL,    "present"},
    {CSS_SEVERITY_CRITICAL,  "configuration error"},
    {CSS_SEVERITY_NORMAL,    "spare"},
    {CSS_SEVERITY_WARNING,   "throttled"},
    {CSS_SEVERITY_CRITICAL,   "overtemperature"}
}; 
    
/* D Drive Bay/Slot */ 
#define SENSOR_SPECIFICTYPED_SIZE (10)   
SensorStateText g_SensorSpecificTypeD[SENSOR_SPECIFICTYPED_SIZE] =
{
    {CSS_SEVERITY_UNKNOWN,   "absent"},  
    {CSS_SEVERITY_NORMAL,    "present and OK"},
    {CSS_SEVERITY_CRITICAL,  "drive fault"},
    {CSS_SEVERITY_WARNING,   "predictive failure"},
    {CSS_SEVERITY_CRITICAL,  "hot spare"},
    {CSS_SEVERITY_WARNING,   "consistency check in progress"},
    {CSS_SEVERITY_CRITICAL,  "critical array"},
    {CSS_SEVERITY_CRITICAL,  "failed array"},
    {CSS_SEVERITY_NORMAL,    "rebuild in progress"},
    {CSS_SEVERITY_WARNING,   "rebuild aborted"}
};
    
/* E POST Memory Resize */ 
#define SENSOR_SPECIFICTYPEE_SIZE (2) 
SensorStateText g_SensorSpecificTypeE[SENSOR_SPECIFICTYPEE_SIZE] =
{
    {CSS_SEVERITY_NORMAL,    "not resized"},  
    {CSS_SEVERITY_NORMAL,    "resized"}        
};
    
/* F POST error */
#define SENSOR_SPECIFICTYPEF_SIZE (4)
SensorStateText g_SensorSpecificTypeF[SENSOR_SPECIFICTYPEF_SIZE] =
{ 
    {CSS_SEVERITY_NORMAL,    "no errors"}, 
    {CSS_SEVERITY_CRITICAL,  "POST error"},
    {CSS_SEVERITY_CRITICAL,  "system firmware hang"},
    {CSS_SEVERITY_CRITICAL,  "system firmware progress"}
};
    
/* 10 event logging */
#define SENSOR_SPECIFICTYPE10_SIZE (8)
SensorStateText g_SensorSpecificType10[SENSOR_SPECIFICTYPE10_SIZE] =
{
    {CSS_SEVERITY_NORMAL,    "good"}, 
    {CSS_SEVERITY_CRITICAL,  "correctable memory error logging disabled"},   
    {CSS_SEVERITY_CRITICAL,  "event logging disabled"},
    {CSS_SEVERITY_NORMAL,    "cleared"},
    {CSS_SEVERITY_CRITICAL,  "all event logging disabled"},
    {CSS_SEVERITY_CRITICAL,  "log full"},
    {CSS_SEVERITY_WARNING,   "log almost full"},
    {CSS_SEVERITY_CRITICAL,  "install error"}
};
    
/* 11 Watchdog1 */  
#define SENSOR_SPECIFICTYPE11_SIZE (9)
SensorStateText g_SensorSpecificType11[SENSOR_SPECIFICTYPE11_SIZE] =
{
    {CSS_SEVERITY_NORMAL,    "no action"},   
    {CSS_SEVERITY_CRITICAL,  "BIOS watchdog reset"},
    {CSS_SEVERITY_CRITICAL,  "OS watchdog reset"},
    {CSS_SEVERITY_CRITICAL,  "OS watchdog shutdown"},
    {CSS_SEVERITY_CRITICAL,  "OS watchdog power down"},
    {CSS_SEVERITY_CRITICAL,  "OS watchdog power cycle"},
    {CSS_SEVERITY_CRITICAL,  "OS watchdog power off"},
    {CSS_SEVERITY_CRITICAL,  "OS watchdog expired"},
    {CSS_SEVERITY_CRITICAL,  "OS watchdog pre-timeout interrupt"}
};  
    
/* 12 System Event */
#define SENSOR_SPECIFICTYPE12_SIZE (7)
SensorStateText g_SensorSpecificType12[SENSOR_SPECIFICTYPE12_SIZE] =
{
    {CSS_SEVERITY_NORMAL,    "none"},
    {CSS_SEVERITY_NORMAL,    "reconfigured"},
    {CSS_SEVERITY_NORMAL,    "OEM system boot"},
    {CSS_SEVERITY_CRITICAL,  "unknown system hardware failure"},
    {CSS_SEVERITY_NORMAL,    "auxilary log entry"},
    {CSS_SEVERITY_NORMAL,    "PEF action executed"},
    {CSS_SEVERITY_NORMAL,    "timestamp clock synch"}
};
    
/* 13 Critical Interrupt */
#define SENSOR_SPECIFICTYPE13_SIZE (12)
SensorStateText g_SensorSpecificType13[SENSOR_SPECIFICTYPE13_SIZE] =
{
    {CSS_SEVERITY_NORMAL,    "no interrupts"}, 
    {CSS_SEVERITY_CRITICAL,  "front panel NMI / diagnostic interrupt"},
    {CSS_SEVERITY_CRITICAL,  "bus timeout"},
    {CSS_SEVERITY_CRITICAL,  "I/O channel check NMI"},
    {CSS_SEVERITY_CRITICAL,  "software NMI"},
    {CSS_SEVERITY_CRITICAL,  "PCI PERR"},
    {CSS_SEVERITY_CRITICAL,  "PCI SERR"},
    {CSS_SEVERITY_CRITICAL,  "EISA fail safe timeout"},
    {CSS_SEVERITY_CRITICAL,  "bus correctable error"},
    {CSS_SEVERITY_CRITICAL,  "bus uncorrectable error"},
    {CSS_SEVERITY_CRITICAL,  "fatal NMI"},
    {CSS_SEVERITY_CRITICAL,  "bus fatal error"}
};
    
/* 14 Button / Switch */
#define SENSOR_SPECIFICTYPE14_SIZE (6)
SensorStateText g_SensorSpecificType14[SENSOR_SPECIFICTYPE14_SIZE] =
{
    {CSS_SEVERITY_NORMAL,    "no action"}, 
    {CSS_SEVERITY_NORMAL,    "power button pressed"},
    {CSS_SEVERITY_NORMAL,    "sleep button pressed"},
    {CSS_SEVERITY_NORMAL,    "reset button pressed"},
    {CSS_SEVERITY_NORMAL,    "FRU latch open"},
    {CSS_SEVERITY_NORMAL,    "FRU service request pressed"}
};

/* 15 Module Board (VRM/RAC etc) currently not in IPMI spec*/
#define SENSOR_SPECIFICTYPE15_SIZE (6)
SensorStateText g_SensorSpecificType15[SENSOR_SPECIFICTYPE15_SIZE] =
{
    {CSS_SEVERITY_UNKNOWN,   "absent"}, 
    {CSS_SEVERITY_NORMAL,    "present"},
    {CSS_SEVERITY_NORMAL,    "function ready"},
    {CSS_SEVERITY_NORMAL,    "full ready"},
    {CSS_SEVERITY_WARNING,   "off line"},
    {CSS_SEVERITY_CRITICAL,  "failed"}
};

    
/* 19 Chip Set */
#define SENSOR_SPECIFICTYPE19_SIZE (2)
SensorStateText g_SensorSpecificType19[SENSOR_SPECIFICTYPE19_SIZE] =
{
    {CSS_SEVERITY_NORMAL,    "good"},
    {CSS_SEVERITY_CRITICAL,  "soft power control failure"}
};

/* 1B Cable / Interconnect */
#define SENSOR_SPECIFICTYPE1B_SIZE (3)
SensorStateText g_SensorSpecificType1B[SENSOR_SPECIFICTYPE1B_SIZE] =
{
    {CSS_SEVERITY_WARNING,   "disconnected"}, 
    {CSS_SEVERITY_NORMAL,    "connected"},
    {CSS_SEVERITY_CRITICAL,  "configuration error"}
};
    
    
/* 1D System Boot */
#define SENSOR_SPECIFICTYPE1D_SIZE (9)
SensorStateText g_SensorSpecificType1D[SENSOR_SPECIFICTYPE1D_SIZE] =
{
    {CSS_SEVERITY_UNKNOWN,   g_UnknownState}, 
    {CSS_SEVERITY_NORMAL,    "power up"},
    {CSS_SEVERITY_NORMAL,    "hard reset"},
    {CSS_SEVERITY_NORMAL,    "warm reset"},
    {CSS_SEVERITY_NORMAL,    "user requested PXE boot"},
    {CSS_SEVERITY_NORMAL,    "automatic boot to diagnostic"},
    {CSS_SEVERITY_NORMAL,    "OS / run-time software initiated hard reset"},
    {CSS_SEVERITY_NORMAL,    "OS / run-time software initiated warm reset"},
    {CSS_SEVERITY_NORMAL,    "system restart"}
};
    
/* 1E Boot Error */
#define SENSOR_SPECIFICTYPE1E_SIZE (9)
SensorStateText g_SensorSpecificType1E[SENSOR_SPECIFICTYPE1E_SIZE] =
{
    {CSS_SEVERITY_NORMAL,    "no errors"},
    {CSS_SEVERITY_CRITICAL,  "no bootable media"},
    {CSS_SEVERITY_CRITICAL,  "non-bootable diskette"},
    {CSS_SEVERITY_CRITICAL,  "PXE server not found"},
    {CSS_SEVERITY_CRITICAL,  "invalid boot sector"},
    {CSS_SEVERITY_CRITICAL,  "timeout waiting for user selection of boot source"}
};
    
/* 1F OS Boot */
#define SENSOR_SPECIFICTYPE1F_SIZE (8)
SensorStateText g_SensorSpecificType1F[SENSOR_SPECIFICTYPE1F_SIZE] =
{
    {CSS_SEVERITY_WARNING,   "no boot"},
    {CSS_SEVERITY_NORMAL,    "A: boot"},
    {CSS_SEVERITY_NORMAL,    "C: boot"},
    {CSS_SEVERITY_NORMAL,    "PXE boot"},
    {CSS_SEVERITY_NORMAL,    "diagnostic boot"},
    {CSS_SEVERITY_NORMAL,    "CD-ROM boot"},
    {CSS_SEVERITY_NORMAL,    "ROM boot"},
    {CSS_SEVERITY_NORMAL,    "booted"}
};
    
/* 20 OS Stop / Shutdown */
#define SENSOR_SPECIFICTYPE20_SIZE (7)
SensorStateText g_SensorSpecificType20[SENSOR_SPECIFICTYPE20_SIZE] =
{
    {CSS_SEVERITY_UNKNOWN,   g_UnknownState},
    {CSS_SEVERITY_CRITICAL,  "critical stop during OS load"},
    {CSS_SEVERITY_CRITICAL,  "run-time critical stop"},
    {CSS_SEVERITY_NORMAL,    "OS graceful stop"},
    {CSS_SEVERITY_NORMAL,    "OS graceful shutdown"},
    {CSS_SEVERITY_CRITICAL,  "soft shutdown initiated by PEF"},
    {CSS_SEVERITY_CRITICAL,  "agent not responding"}
};
    
/* 21 Slot */
#define SENSOR_SPECIFICTYPE21_SIZE (11)
SensorStateText g_SensorSpecificType21[SENSOR_SPECIFICTYPE21_SIZE] =
{
    {CSS_SEVERITY_UNKNOWN,   g_UnknownState},                
    {CSS_SEVERITY_CRITICAL,  "fault"},
    {CSS_SEVERITY_NORMAL,    "identify"},
    {CSS_SEVERITY_NORMAL,    "installed"},
    {CSS_SEVERITY_NORMAL,    "ready for installation"},
    {CSS_SEVERITY_NORMAL,    "ready for removal"},
    {CSS_SEVERITY_NORMAL,    "power is off"},
    {CSS_SEVERITY_NORMAL,    "removal requested"},
    {CSS_SEVERITY_NORMAL,    "interlocked"},
    {CSS_SEVERITY_WARNING,   "disabled"},
    {CSS_SEVERITY_NORMAL,    "holds spare device"}
};
    
/* 22 System ACPI Power State */
#define SENSOR_SPECIFICTYPE22_SIZE (14)
SensorStateText g_SensorSpecificType22[SENSOR_SPECIFICTYPE22_SIZE] =
{
    {CSS_SEVERITY_UNKNOWN,   g_UnknownState},
    {CSS_SEVERITY_NORMAL,    "S0/G0"},
    {CSS_SEVERITY_NORMAL,    "S1"},
    {CSS_SEVERITY_NORMAL,    "S2"},
    {CSS_SEVERITY_NORMAL,    "S3"},
    {CSS_SEVERITY_NORMAL,    "S4"},
    {CSS_SEVERITY_NORMAL,    "S5/G2"},
    {CSS_SEVERITY_NORMAL,    "S4/S5"},
    {CSS_SEVERITY_NORMAL,    "G3"},
    {CSS_SEVERITY_NORMAL,    "sleeping in an S1, S2, or S3 states"},
    {CSS_SEVERITY_NORMAL,    "G1"},
    {CSS_SEVERITY_NORMAL,    "S5 entered by override"},
    {CSS_SEVERITY_NORMAL,    "legacy ON"},
    {CSS_SEVERITY_NORMAL,    "legacy OFF"}
};
    
/* 23 Watchdog 2 */
#define SENSOR_SPECIFICTYPE23_SIZE (10)
SensorStateText g_SensorSpecificType23[SENSOR_SPECIFICTYPE23_SIZE] =
{
    {CSS_SEVERITY_NORMAL,    "none"},
    {CSS_SEVERITY_CRITICAL,  "timer expired"},
    {CSS_SEVERITY_CRITICAL,  "rebooted"},
    {CSS_SEVERITY_CRITICAL,  "power off"},
    {CSS_SEVERITY_CRITICAL,  "power cycle"},
    {CSS_SEVERITY_UNKNOWN,   g_UnknownState},        
    {CSS_SEVERITY_UNKNOWN,   g_UnknownState},        
    {CSS_SEVERITY_UNKNOWN,   g_UnknownState},        
    {CSS_SEVERITY_UNKNOWN,   g_UnknownState},        
    {CSS_SEVERITY_CRITICAL,  "timer interrupt"}
};
    
/* 24 Platform Alert */
#define SENSOR_SPECIFICTYPE24_SIZE (5)
SensorStateText g_SensorSpecificType24[SENSOR_SPECIFICTYPE24_SIZE] =
{
    {CSS_SEVERITY_NORMAL,    "none"},
    {CSS_SEVERITY_CRITICAL,  "page"},
    {CSS_SEVERITY_CRITICAL,  "LAN alert"},
    {CSS_SEVERITY_CRITICAL,  "event trap"},
    {CSS_SEVERITY_CRITICAL,  "SNMP trap"}
};
    
/* 25 Entity Presence */
#define SENSOR_SPECIFICTYPE25_SIZE (4)
SensorStateText g_SensorSpecificType25[SENSOR_SPECIFICTYPE25_SIZE] =
{
    {CSS_SEVERITY_UNKNOWN,   g_UnknownState},            
    {CSS_SEVERITY_NORMAL,    "present"},
    {CSS_SEVERITY_CRITICAL,  "absent"},
    {CSS_SEVERITY_NORMAL ,   "disabled"}
};

/* 27 LAN */
#define SENSOR_SPECIFICTYPE27_SIZE (3)
SensorStateText g_SensorSpecificType27[SENSOR_SPECIFICTYPE27_SIZE] =
{
    {CSS_SEVERITY_UNKNOWN,   g_UnknownState},
    {CSS_SEVERITY_CRITICAL,  "heartbeat lost"},
    {CSS_SEVERITY_NORMAL,    "heartbeat"}
};

/* 28 Management Subsystem Health */
#define SENSOR_SPECIFICTYPE28_SIZE (8)
SensorStateText g_SensorSpecificType28[SENSOR_SPECIFICTYPE28_SIZE] =
{
    {CSS_SEVERITY_UNKNOWN,   g_UnknownState},
    {CSS_SEVERITY_NORMAL,    "on-line"},
    {CSS_SEVERITY_WARNING,   "sensor access degraded or unavailable"},
    {CSS_SEVERITY_WARNING,   "controller access degraded or unavailable"},
    {CSS_SEVERITY_WARNING,   "management controller off-line"},
    {CSS_SEVERITY_WARNING,   "management controller unavailable"},
    {CSS_SEVERITY_CRITICAL,  "sensor failure"},
    {CSS_SEVERITY_CRITICAL,  "FRU failure"}
};

/* 29 Battery */
#define SENSOR_SPECIFICTYPE29_SIZE (4)
SensorStateText g_SensorSpecificType29[SENSOR_SPECIFICTYPE29_SIZE] =
{
    {CSS_SEVERITY_UNKNOWN,   "absent"},  
    {CSS_SEVERITY_WARNING,   "low"},
    {CSS_SEVERITY_CRITICAL,  "failed"},
    {CSS_SEVERITY_NORMAL,    "good"}
};    
    
/* 2A Session Audit */
#define SENSOR_SPECIFICTYPE2A_SIZE (3)
SensorStateText g_SensorSpecificType2A[SENSOR_SPECIFICTYPE2A_SIZE] =
{
    {CSS_SEVERITY_NORMAL,    "none"},  
    {CSS_SEVERITY_NORMAL,    "activated"},
    {CSS_SEVERITY_NORMAL,    "deactivated"}
};
    
/* 2B Version Change */
#define SENSOR_SPECIFICTYPE2B_SIZE (9)
SensorStateText g_SensorSpecificType2B[SENSOR_SPECIFICTYPE2B_SIZE] =
{
    {CSS_SEVERITY_NORMAL,    "none"},  
    {CSS_SEVERITY_WARNING,   "hardware change"},
    {CSS_SEVERITY_WARNING,   "firmware or software change"},
    {CSS_SEVERITY_CRITICAL,  "hardware incompatibility"},
    {CSS_SEVERITY_CRITICAL,  "firmware or software incompatibility"},
    {CSS_SEVERITY_CRITICAL,  "invalid/unsupported hardware version"},
    {CSS_SEVERITY_CRITICAL,  "invalid/unsupported firmware/software version"},
    {CSS_SEVERITY_NORMAL,    "hardware change was successful"},
    {CSS_SEVERITY_NORMAL,    "software or F/W change was successful"}
};
    
/* 2C FRU State */
#define SENSOR_SPECIFICTYPE2C_SIZE (9)
SensorStateText g_SensorSpecificType2C[SENSOR_SPECIFICTYPE2C_SIZE] =
{
    {CSS_SEVERITY_UNKNOWN,   g_UnknownState},  
    {CSS_SEVERITY_NORMAL,    "not installed"},
    {CSS_SEVERITY_NORMAL,    "inactive"},
    {CSS_SEVERITY_NORMAL,    "activation requested"},
    {CSS_SEVERITY_NORMAL,    "activation in progress"},
    {CSS_SEVERITY_NORMAL,    "active"},
    {CSS_SEVERITY_WARNING,   "deactivation requested"},
    {CSS_SEVERITY_WARNING,   "deactivation in progress"},
    {CSS_SEVERITY_CRITICAL,  "communication lost"}
};

/* Specific Assert access Table */
SensorStateElement g_SensorSpecificTable[] =
{
    {IPMI_SENSOR_INTRUSION,        SENSOR_SPECIFICTYPE5_SIZE,  g_SensorSpecificType5},
    {IPMI_SENSOR_SECURE_MODE,      SENSOR_SPECIFICTYPE6_SIZE,  g_SensorSpecificType6},
    {IPMI_SENSOR_PROCESSOR,        SENSOR_SPECIFICTYPE7_SIZE, g_SensorSpecificType7},
    {IPMI_SENSOR_PS,               SENSOR_SPECIFICTYPE8_SIZE,  g_SensorSpecificType8},
    {IPMI_SENSOR_POWER_UNIT,       SENSOR_SPECIFICTYPE9_SIZE,  g_SensorSpecificType9},
    {IPMI_SENSOR_MEMORY,           SENSOR_SPECIFICTYPEC_SIZE, g_SensorSpecificTypeC},
    {IPMI_SENSOR_DRIVE_SLOT,       SENSOR_SPECIFICTYPED_SIZE,  g_SensorSpecificTypeD},
    {IPMI_SENSOR_CRIT_EVENT,       SENSOR_SPECIFICTYPE13_SIZE, g_SensorSpecificType13},
    {IPMI_SENSOR_CHIP_SET,         SENSOR_SPECIFICTYPE19_SIZE,  g_SensorSpecificType19},
    {IPMI_SENSOR_SLOT,             SENSOR_SPECIFICTYPE21_SIZE, g_SensorSpecificType21},
    {IPMI_SENSOR_BATTERY,          SENSOR_SPECIFICTYPE29_SIZE,  g_SensorSpecificType29},
    {IPMI_SENSOR_MEM_RESIZE,       SENSOR_SPECIFICTYPEE_SIZE,  g_SensorSpecificTypeE},
    {IPMI_SENSOR_POST_ERROR,       SENSOR_SPECIFICTYPEF_SIZE,  g_SensorSpecificTypeF},
    {IPMI_SENSOR_EVENT_LOGGING,    SENSOR_SPECIFICTYPE10_SIZE,  g_SensorSpecificType10},
    {IPMI_SENSOR_WDOG,             SENSOR_SPECIFICTYPE11_SIZE,  g_SensorSpecificType11},
    {IPMI_SENSOR_SYSTEM_EVENT,     SENSOR_SPECIFICTYPE12_SIZE,  g_SensorSpecificType12},
    {IPMI_SENSOR_POWER_BUTTON,     SENSOR_SPECIFICTYPE14_SIZE,  g_SensorSpecificType14},
    {IPMI_SENSOR_VRM,              SENSOR_SPECIFICTYPE15_SIZE,  g_SensorSpecificType15},
    {IPMI_SENSOR_CI,               SENSOR_SPECIFICTYPE1B_SIZE,  g_SensorSpecificType1B},
    {IPMI_SENSOR_SYSTEM_BOOT,      SENSOR_SPECIFICTYPE1D_SIZE,  g_SensorSpecificType1D},
    {IPMI_SENSOR_BOOT_ERROR,       SENSOR_SPECIFICTYPE1E_SIZE,  g_SensorSpecificType1E},
    {IPMI_SENSOR_OS_BOOT,          SENSOR_SPECIFICTYPE1F_SIZE,  g_SensorSpecificType1F},
    {IPMI_SENSOR_OS_CRITICAL_STOP, SENSOR_SPECIFICTYPE20_SIZE,  g_SensorSpecificType20},
    {IPMI_SENSOR_ACPI,             SENSOR_SPECIFICTYPE22_SIZE, g_SensorSpecificType22},
    {IPMI_SENSOR_WDOG2,            SENSOR_SPECIFICTYPE23_SIZE,  g_SensorSpecificType23},
    {IPMI_SENSOR_PLATFORM_ALERT,   SENSOR_SPECIFICTYPE24_SIZE,  g_SensorSpecificType24},
    {IPMI_SENSOR_ENTITY_PRESENCE,  SENSOR_SPECIFICTYPE25_SIZE,  g_SensorSpecificType25},
    {IPMI_SENSOR_LAN,              SENSOR_SPECIFICTYPE27_SIZE,  g_SensorSpecificType27},
    {IPMI_SENSOR_MGMT_SYS_HEALTH,  SENSOR_SPECIFICTYPE28_SIZE,  g_SensorSpecificType28},    
    {IPMI_SENSOR_SESSION_AUDIT,    SENSOR_SPECIFICTYPE2A_SIZE,  g_SensorSpecificType2A},
    {IPMI_SENSOR_VERSION_CHANGE,   SENSOR_SPECIFICTYPE2B_SIZE,  g_SensorSpecificType2B},
    {IPMI_SENSOR_FRU_STATE,        SENSOR_SPECIFICTYPE2C_SIZE,  g_SensorSpecificType2C}
};


